import inspect
from typing import (
    Callable,
    Optional,
)
import uuid

from PySide6.QtWidgets import QPushButton
from PySide6.QtCore import Qt, QPropertyAnimation, QEasingCurve, QPoint, Property, QObject, QTimer, Signal
from PySide6.QtGui import QPainter, QBrush, QPainterPath

from qtmui.hooks import State

from ..system.color_manipulator import get_palette_text_color, hex_string_to_qcolor
# from ..tooltip import ToolTipFilter
from ..widget_base import PyWidgetBase

from qtmui.material.styles import useTheme

class RippleEffect(QObject):
    """Ripple effect handler for button interactions."""
    def __init__(self, parent=None):
        super().__init__(parent)
        self._radius = 0
        self._max_radius = 100
        self._opacity = 1.0
        self._center = QPoint(0, 0)

    def radius(self):
        return self._radius

    def setRadius(self, radius):
        self._radius = radius
        self.parent().update()

    def opacity(self):
        return self._opacity

    def setOpacity(self, opacity):
        self._opacity = opacity
        self.parent().update()

    def center(self):
        return self._center

    def setCenter(self, center):
        self._center = center

    radius = Property(int, fget=radius, fset=setRadius)
    opacity = Property(float, fget=opacity, fset=setOpacity)
    center = Property(QPoint, fget=center, fset=setCenter)

class ButtonBase(QPushButton, PyWidgetBase):
    clicked = Signal()
    mouseEnterSignal = Signal()
    mouseLeaverSignal = Signal()

    """
    Button cơ bản với hỗ trợ các sự kiện tương tác, hiệu ứng ripple, và các thuộc tính khác.
    """

    def __init__(
        self,
        id: str = None,  # ===========> xem lại
        key: str = None,  # ===========> xem lại
        value: object = None,  # ===========> xem lại
        action: Optional[Callable] = None,  # Tham chiếu hành động bắt buộc. Hiện chỉ hỗ trợ hành động focusVisible().
        centerRipple: bool = False,  # Nếu là True, hiệu ứng ripple sẽ được đặt ở trung tâm. Nó sẽ không bắt đầu tại vị trí tương tác của chuột.
        children: Optional[object] = None,  # Nội dung của component.
        classes: Optional[dict] = None,  # Ghi đè hoặc mở rộng các kiểu được áp dụng cho component.
        component: Optional[object] = None,  # Component sử dụng cho nút gốc. Có thể là chuỗi tên thẻ HTML hoặc một component khác.
        disabled: bool = False,  # Nếu là True, component sẽ bị vô hiệu hóa.
        disableRipple: bool = False,  # Nếu là True, hiệu ứng ripple sẽ bị vô hiệu hóa.
        disableTouchRipple: bool = False,  # Nếu là True, hiệu ứng ripple khi chạm sẽ bị vô hiệu hóa.
        focusRipple: bool = False,  # Nếu là True, button cơ bản sẽ có hiệu ứng ripple khi focus bằng bàn phím.
        focusVisibleClassName: Optional[str] = None,  # Thuộc tính giúp xác định phần tử nào đang có focus từ bàn phím.
        LinkComponent: Optional[object] = None,  # Component dùng để render liên kết khi thuộc tính href được cung cấp.
        onFocusVisible: Optional[Callable] = None,  # Callback gọi khi component được focus bằng bàn phím.
        sx: Optional[object] = None,  # Thuộc tính hệ thống cho phép định nghĩa override hệ thống và các kiểu CSS bổ sung.
        tabIndex: Optional[int] = 0,  # Chỉ số tab để quản lý thứ tự focus khi điều hướng bằng bàn phím.
        TouchRippleProps: Optional[dict] = None,  # Các thuộc tính được áp dụng cho phần tử TouchRipple.
        touchRippleRef: Optional[Callable] = None,  # Tham chiếu tới phần tử TouchRipple.
        tooltip: Optional[str] = None,  # Tham chiếu tới phần tử TouchRipple.
        tooltipMaxWidth: Optional[int] = 300,  # Tham chiếu tới phần tử TouchRipple.
        tooltipPlacement: Optional[str] = "bottom",  # Tham chiếu tới phần tử TouchRipple.
        onBlur: Optional[Callable] = None,  # Callback khi sự kiện mất focus diễn ra.
        onClick: Optional[Callable] = None,  # Callback khi sự kiện click diễn ra.
        onMouseEnter: Optional[Callable] = None,  # Callback khi sự kiện click diễn ra.
        onContextMenu: Optional[Callable] = None,  # Callback khi sự kiện context menu (nhấn chuột phải) diễn ra.
        onFocus: Optional[Callable] = None,  # Callback khi sự kiện focus diễn ra.
        onKeyDown: Optional[Callable] = None,  # Callback khi sự kiện nhấn phím diễn ra.
        onKeyUp: Optional[Callable] = None,  # Callback khi sự kiện nhả phím diễn ra.
        onMouseDown: Optional[Callable] = None,  # Callback khi sự kiện nhấn chuột diễn ra.
        onMouseLeave: Optional[Callable] = None,  # Callback khi sự kiện di chuột ra khỏi component diễn ra.
        onMouseUp: Optional[Callable] = None,  # Callback khi sự kiện nhả chuột diễn ra.
        onDragLeave: Optional[Callable] = None,  # Callback khi sự kiện kéo thả chuột ra khỏi component diễn ra.
        rippleDuration: int = 600,  # Callback khi sự kiện kéo thả chuột ra khỏi component diễn ra.
        rippleTimeout: int = None,  # Callback khi sự kiện kéo thả chuột ra khỏi component diễn ra.
        
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        PyWidgetBase._setUpUi(self)

        # gán giá trị truyền vào cho các thuộc tính của ButtonBase
        self._value = value
        
        self._action = action
        self._centerRipple = centerRipple
        self._children = children
        self._component = component
        self._classes = classes
        self._disabled = disabled
        self._disableRipple = disableRipple
        self._disableTouchRipple = disableTouchRipple
        self._focusRipple = focusRipple
        self._focusVisibleClassName = focusVisibleClassName
        self._LinkComponent = LinkComponent
        self._onFocusVisible = onFocusVisible
        self._sx = sx
        self._tabIndex = tabIndex
        self._TouchRippleProps = TouchRippleProps
        self._touchRippleRef = touchRippleRef
        self._tooltip = tooltip
        self._tooltipPlacement = tooltipPlacement
        self._rippleDuration = rippleDuration
        self._rippleTimeout = rippleTimeout

        # Gán các callback cho sự kiện tương tác
        self._onBlur = onBlur
        self._onClick = onClick
        self._onMouseEnter = onMouseEnter
        self._onContextMenu = onContextMenu
        self._onFocus = onFocus
        self._onKeyDown = onKeyDown
        self._onKeyUp = onKeyUp
        self._onMouseDown = onMouseDown
        self._onMouseLeave = onMouseLeave
        self._onMouseUp = onMouseUp
        self._onDragLeave = onDragLeave


        # ripple config
        self._color = "#333333"

        self._radius = 0
        self._max_radius = 100
        self._opacity = 1.0
        self._center = QPoint(0, 0)

        self._setup_ui()


    def _setup_ui(self):
        # ripple effect config
        # if not self._disableRipple:
        #     self.ripple_effect = RippleEffect(self)
        #     self.animation = QPropertyAnimation(self.ripple_effect, b"radius")
        #     self.opacity_animation = QPropertyAnimation(self.ripple_effect, b"opacity")
        #     self.animation.setEasingCurve(QEasingCurve.OutQuad)
        #     self.opacity_animation.setEasingCurve(QEasingCurve.OutQuad)

        if self._tooltip:
            PyWidgetBase._installTooltipFilter(self)

        if self._onClick:
            sig = inspect.signature(self._onClick)
            if "sender" in sig.parameters:
                self.clicked.connect(lambda *args, **kwargs: self._onClick(sender=self, *args, **kwargs))
            else:
                self.clicked.connect(lambda *args, **kwargs: self._onClick(*args, **kwargs))


        if self._onMouseEnter:
            self.mouseEnterSignal.connect(self._onMouseEnter)

        if self._onMouseLeave:
            self.mouseLeaverSignal.connect(self._onMouseLeave)
        

        # self._connect_signals_to_slots()

        # self.installEventFilter(self)

    def _connect_signals_to_slots(self):
        if isinstance(self._disabled, State):
            self._disabled.valueChanged.connect(self._set_enabled)
            self.setEnabled(not self._disabled.value)
        elif isinstance(self._disabled, bool):
            self.setEnabled(not self._disabled)

    def _set_enabled(self, state):
        self.setEnabled(not state)

    def _run_ripple_animation(self):
        self.animation.setStartValue(0)
        self.animation.setEndValue(self.ripple_effect._max_radius)
        self.animation.setDuration(self._rippleDuration)

        self.opacity_animation.setStartValue(1.0)
        self.opacity_animation.setEndValue(0.0)
        self.opacity_animation.setDuration(self._rippleDuration)

        self.animation.start()
        self.opacity_animation.start()

    # Hàm này cho phép bạn đăng ký một callback onClick
    def set_on_click(self, callback):
        self._onClick = callback

    # def focusInEvent(self, event):
    #     if self._onFocus:
    #         self._onFocus(event)
    #     super().focusInEvent(event)

    # def focusOutEvent(self, event):
    #     if self._onBlur:
    #         self._onBlur(event)
    #     super().focusOutEvent(event)

    # def contextMenuEvent(self, event):
    #     if self._onContextMenu:
    #         self._onContextMenu(event)
    #     super().contextMenuEvent(event)

    def enterEvent(self, event):
        if self._onMouseEnter:
            self.mouseEnterSignal.emit()
        return super().enterEvent(event)

    def leaveEvent(self, event):
        if self._onMouseLeave:
            self.mouseLeaverSignal.emit()

    #     super().leaveEvent(event)

    # def mousePressEvent(self, event):
    #     """
    #     Override mousePressEvent to handle ripple effect and trigger the ripple animation.
    #     """
    #     if hasattr(self, "ripple_effect") and not self._disableRipple:
    #         self.ripple_effect.setCenter(event.pos())
    #         self.ripple_effect._max_radius = max(self.width(), self.height())

    #         if self._rippleTimeout:
    #             QTimer.singleShot(self._rippleTimeout, self._run_ripple_animation)
    #         else:
    #             self._run_ripple_animation()

    #     if self._onMouseDown:
    #         self._onMouseDown(event)

    #     super().mousePressEvent(event)


    # def mouseReleaseEvent(self, event):
    #     if event.button() == Qt.LeftButton: # neu đặt ở đây thì hai sự kiện click được kích hoạt ====> k đúng, và Widget có hàm self.onClickHandle thì ko có self.sender
    #         # self._onClick([event, self._value])
    #         # self._onClick([event, self._value])
    #         # self.clicked.emit()
    #         # self._onClick()
    #         pass
    #     super().mouseReleaseEvent(event)

    # def mouseMoveEvent(self, event):
    #     super().mouseMoveEvent(event)

    # def keyPressEvent(self, event):
    #     if self._onKeyDown:
    #         self._onKeyDown(event)
    #     super().keyPressEvent(event)

    # def keyReleaseEvent(self, event):
    #     if self._onKeyUp:
    #         self._onKeyUp(event)
    #     super().keyReleaseEvent(event)

    # def dragLeaveEvent(self, event):
    #     if self._onDragLeave:
    #         self._onDragLeave(event)
    #     super().dragLeaveEvent(event)

    # def showEvent(self, e): # thêm vào đây nó nháy bỏ mẹ ra
    #     """ fade in """
    #     PyWidgetBase.showEvent(self)
    #     super().showEvent(e)

    # def changeEvent(self, e):
    #     PyWidgetBase.changeEvent(self, e)
    #     return super().changeEvent(e)

    # def paintEvent(self, event):
    #     if not hasattr(self, "theme") or not self.theme:
    #         self.theme = useTheme()
    #     """Handle the painting of the button, including ripple effect if enabled."""
    #     if hasattr(self, "ripple_effect") and not self._disableRipple and self.ripple_effect.opacity > 0:
    #         try:
    #             painter = QPainter(self)
    #             painter.setRenderHint(QPainter.RenderHint.Antialiasing, True)
    #             painter.setPen(Qt.NoPen)

    #             clip_path = QPainterPath()
    #             if self._component == "Radio" or self._component == "Checkbox" or self._component == "IconButton":
    #                 clip_path.addRoundedRect(0, 0, self.width(), self.height(), self.width()/2, self.height()/2)
    #             else:
    #                 clip_path.addRoundedRect(0, 0, self.width(), self.height(), 4, 4)

    #             painter.setClipPath(clip_path)

    #             color = hex_string_to_qcolor(get_palette_text_color(self.theme.palette, self._color))
    #             painter.setBrush(QBrush(color))
    #             center = self.ripple_effect.center
    #             painter.drawEllipse(center, self.ripple_effect.radius, self.ripple_effect.radius)

    #             # Kết thúc vẽ
    #             painter.end()
    #         except Exception as e:
    #             print('button_base -> eventFilter', str(e))
    #     super().paintEvent(event)



    # def eventFilter(self, obj, event):
    #     if event.type() == QEvent.MouseButtonPress:
    #         # if not self.timer.isActive():
    #         #   self.timer.start()
    #         if event.button() == Qt.LeftButton:
    #             if self._onClick:
    #                 # self._onClick(self)
    #                 pass

    #     return super().eventFilter(obj, event)


    #   elif event.type() == QEvent.MouseButtonDblClick:
    #     self.is_double = True
    #     return True

    #   if event.type() == QEvent.MouseMove:
    #     index = event.pos()
    #   elif event.type() == QEvent.Leave:
    #     index = QModelIndex()
    #   elif event.type() == QEvent.MouseButtonRelease:
    #     index = QModelIndex()


import uuid
from typing import Optional, Union, Dict, Callable, List, Literal
from PySide6.QtCore import Signal, Qt, QPropertyAnimation, QRect, QEasingCurve, QTimer, QSize
from PySide6.QtGui import QColor, QKeyEvent
from PySide6.QtWidgets import QVBoxLayout, QSizePolicy, QApplication, QScrollArea, QWidget
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.widget_base.anim_manager import AnimManager
from qtmui.material.system.color_manipulator import alpha

from qtmui.hooks import State

from .dialog_base import DialogBase
from ...material.styles import useTheme
from ..widget_base import PyWidgetBase
from ..utils.validate_params import _validate_param

class Dialog(DialogBase, PyWidgetBase, AnimManager):
    """
    A dialog component that displays modal or non-modal content with transitions.

    The `Dialog` component is used to display content in a modal or non-modal dialog,
    supporting all props of the Material-UI `Dialog` and `Modal` components, as well
    as additional props for customization.

    Parameters
    ----------
    open : State or bool, optional
        If True, the dialog is shown. Default is False.
        Can be a `State` object for dynamic updates.
    ariaDescribedby : State, str, or None, optional
        The id(s) of the element(s) that describe the dialog. Default is None.
        Can be a `State` object for dynamic updates.
    ariaLabelledby : State, str, or None, optional
        The id(s) of the element(s) that label the dialog. Default is None.
        Can be a `State` object for dynamic updates.
    ariaModal : State or bool, optional
        Informs assistive technologies that the element is modal. Default is True.
        Can be a `State` object for dynamic updates.
    BackdropComponent : State, str, or None, optional
        The component used for the backdrop. Default is None (uses default backdrop).
        Can be a `State` object for dynamic updates. Deprecated, use slots.backdrop instead.
    children : State, QWidget, List[QWidget], or None, optional
        The dialog children, usually sub-components. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    disableEscapeKeyDown : State or bool, optional
        If True, hitting Escape will not fire onClose. Default is False.
        Can be a `State` object for dynamic updates.
    fullScreen : State or bool, optional
        If True, the dialog is full-screen. Default is False.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the dialog stretches to maxWidth. Default is False.
        Can be a `State` object for dynamic updates.
    maxWidth : State, str, or bool, optional
        Determines the max-width of the dialog ("xs", "sm", "md", "lg", "xl", False).
        Default is "sm". Can be a `State` object for dynamic updates.
    onClose : State, Callable, or None, optional
        Callback fired when the dialog requests to be closed.
        Signature: function(event: object, reason: string) -> None
        Default is None. Can be a `State` object for dynamic updates.
    PaperComponent : State, str, or None, optional
        The component used for the dialog body. Default is None (uses default paper).
        Can be a `State` object for dynamic updates. Deprecated, use slots.paper instead.
    PaperProps : State or dict, optional
        Props applied to the Paper element. Default is None.
        Can be a `State` object for dynamic updates. Deprecated, use slotProps.paper instead.
    scroll : State or str, optional
        Determines the container for scrolling ("body" or "paper"). Default is "paper".
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (backdrop, container, paper, root, transition). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for each slot (backdrop, container, paper, root, transition). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    TransitionComponent : State, str, or None, optional
        The component used for the transition (e.g., "Fade"). Default is None (uses Fade).
        Can be a `State` object for dynamic updates. Deprecated, use slots.transition instead.
    transitionDuration : State, int, dict, or None, optional
        The duration for the transition (ms or {appear, enter, exit}). Default is None.
        Can be a `State` object for dynamic updates.
    TransitionProps : State or dict, optional
        Props applied to the transition element. Default is None.
        Can be a `State` object for dynamic updates. Deprecated, use slotProps.transition instead.
    parent : QWidget or None, optional
        The parent widget. Default is None (uses mainWindow).
    borderRadius : State or int, optional
        The border radius of the dialog. Default is 10.
        Can be a `State` object for dynamic updates.
    onButtonOkClicked : State, Callable, or None, optional
        Callback fired when the OK button is clicked. Default is None.
        Can be a `State` object for dynamic updates.
    width : State, int, or None, optional
        The fixed width of the dialog. Default is None.
        Can be a `State` object for dynamic updates.
    height : State, int, or None, optional
        The fixed height of the dialog. Default is None.
        Can be a `State` object for dynamic updates.
    title : State, QWidget, str, or None, optional
        The title of the dialog. Default is None.
        Can be a `State` object for dynamic updates.
    transition : State, str, or None, optional
        The transition effect (e.g., "fadeIn"). Default is None.
        Can be a `State` object for dynamic updates.

    Attributes
    ----------
    customSignal : Signal
        Custom signal for dialog events.
    VALID_MAX_WIDTH : list[Union[str, bool]]
        Valid values for `maxWidth`: ["xs", "sm", "md", "lg", "xl", False].
    VALID_SCROLL : list[str]
        Valid values for `scroll`: ["body", "paper"].

    Notes
    -----
    - Props of the `Modal` component are supported (e.g., `open`, `onClose`).
    - Deprecated props (`BackdropComponent`, `PaperComponent`, `PaperProps`, `TransitionComponent`,
      `TransitionProps`) are supported but should be replaced with `slots` and `slotProps`.
    - The `transition` prop is mapped to `TransitionComponent` for compatibility.

    Demos:
    - Dialog: https://qtmui.com/material-ui/qtmui-dialog/

    API Reference:
    - Dialog API: https://qtmui.com/material-ui/api/dialog/
    - Modal API: https://qtmui.com/material-ui/api/modal/
    """

    customSignal = Signal()
    VALID_MAX_WIDTH = ["xs", "sm", "md", "lg", "xl", False]
    VALID_SCROLL = ["body", "paper"]

    def __init__(
        self,
        open: Union[State, bool] = False,
        ariaDescribedby: Optional[Union[State, str]] = None,
        ariaLabelledby: Optional[Union[State, str]] = None,
        ariaModal: Union[State, bool] = True,
        BackdropComponent: Optional[Union[State, str]] = None,
        children: Optional[Union[State, Callable, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        disableEscapeKeyDown: Union[State, bool] = False,
        fullScreen: Union[State, bool] = False,
        fullWidth: Union[State, bool] = False,
        maxWidth: Union[State, str, bool] = 300, # "sm"
        onClose: Optional[Union[State, Callable]] = None,
        PaperComponent: Optional[Union[State, str]] = None,
        PaperProps: Optional[Union[State, Dict]] = None,
        scroll: Union[State, str] = "paper",
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        TransitionComponent: Optional[Union[State, str]] = None,
        transitionDuration: Optional[Union[State, int, Dict[str, int]]] = None,
        TransitionProps: Optional[Union[State, Dict]] = None,
        parent: Optional[QWidget] = None,
        borderRadius: Union[State, int] = 10,
        onButtonOkClicked: Optional[Union[State, Callable]] = None,
        width: Optional[Union[State, int]] = None,
        height: Optional[Union[State, int]] = None,
        title: Optional[Union[State, QWidget, str]] = None,
        transition: Optional[Union[State, str]] = None,
        **kwargs
    ):
        super().__init__(parent or QApplication.instance().mainWindow)
        self.setObjectName("PyDialog")
        # self.widget.setObjectName(str(uuid.uuid4()))
        PyWidgetBase._setUpUi(self)
        AnimManager.__init__(self.widget, **kwargs)
        
        self.hide()
        
        # self.setParent(parent)

        # Set properties with validation
        self._set_open(open)
        self._set_aria_describedby(ariaDescribedby)
        self._set_aria_labelledby(ariaLabelledby)
        self._set_aria_modal(ariaModal)
        self._set_BackdropComponent(BackdropComponent)
        self._set_children(children)
        self._set_classes(classes)
        self._set_disableEscapeKeyDown(disableEscapeKeyDown)
        self._set_fullScreen(fullScreen)
        self._set_fullWidth(fullWidth)
        self._set_maxWidth(maxWidth)
        self._set_onClose(onClose)
        self._set_PaperComponent(PaperComponent)
        self._set_PaperProps(PaperProps)
        self._set_scroll(scroll)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_TransitionComponent(TransitionComponent)
        self._set_transitionDuration(transitionDuration)
        self._set_TransitionProps(TransitionProps)
        self._set_parent(parent)
        self._set_borderRadius(borderRadius)
        self._set_onButtonOkClicked(onButtonOkClicked)
        self._set_width(width)
        self._set_height(height)
        self._set_title(title)
        self._set_transition(transition)

        self._animation = None
    
        self.__init_ui()


    # Setter and Getter methods for all parameters
    # @_validate_param(file_path="qtmui.material.dialog", param_name="open", supported_signatures=Union[State, bool])
    def _set_open(self, value):
        self._open = value

    def _get_open(self):
        return self._open.value if isinstance(self._open, State) else self._open

    @_validate_param(file_path="qtmui.material.dialog", param_name="ariaDescribedby", supported_signatures=Union[State, str, type(None)])
    def _set_aria_describedby(self, value):
        self._aria_describedby = value

    def _get_aria_describedby(self):
        return self._aria_describedby.value if isinstance(self._aria_describedby, State) else self._aria_describedby

    @_validate_param(file_path="qtmui.material.dialog", param_name="ariaLabelledby", supported_signatures=Union[State, str, type(None)])
    def _set_aria_labelledby(self, value):
        self._aria_labelledby = value

    def _get_aria_labelledby(self):
        return self._aria_labelledby.value if isinstance(self._aria_labelledby, State) else self._aria_labelledby

    @_validate_param(file_path="qtmui.material.dialog", param_name="ariaModal", supported_signatures=Union[State, bool])
    def _set_aria_modal(self, value):
        self._aria_modal = value

    def _get_aria_modal(self):
        return self._aria_modal.value if isinstance(self._aria_modal, State) else self._aria_modal

    @_validate_param(file_path="qtmui.material.dialog", param_name="BackdropComponent", supported_signatures=Union[State, str, type(None)])
    def _set_BackdropComponent(self, value):
        self._BackdropComponent = value

    def _get_BackdropComponent(self):
        return self._BackdropComponent.value if isinstance(self._BackdropComponent, State) else self._BackdropComponent

    # @_validate_param(file_path="qtmui.material.dialog", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        # if isinstance(value, list) and not all(isinstance(item, QWidget) for item in value if item is not None):
        #     raise ValueError("children must be a QWidget or a list of QWidgets")
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.dialog", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.dialog", param_name="disableEscapeKeyDown", supported_signatures=Union[State, bool])
    def _set_disableEscapeKeyDown(self, value):
        self._disableEscapeKeyDown = value

    def _get_disableEscapeKeyDown(self):
        return self._disableEscapeKeyDown.value if isinstance(self._disableEscapeKeyDown, State) else self._disableEscapeKeyDown

    @_validate_param(file_path="qtmui.material.dialog", param_name="fullScreen", supported_signatures=Union[State, bool])
    def _set_fullScreen(self, value):
        self._fullScreen = value

    def _get_fullScreen(self):
        return self._fullScreen.value if isinstance(self._fullScreen, State) else self._fullScreen

    @_validate_param(file_path="qtmui.material.dialog", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        self._fullWidth = value

    def _get_fullWidth(self):
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    # @_validate_param(file_path="qtmui.material.dialog", param_name="maxWidth", supported_signatures=Union[State, str, bool], valid_values=VALID_MAX_WIDTH)
    def _set_maxWidth(self, value):
        self._maxWidth = value

    def _get_maxWidth(self):
        return self._maxWidth.value if isinstance(self._maxWidth, State) else self._maxWidth

    @_validate_param(file_path="qtmui.material.dialog", param_name="onClose", supported_signatures=Union[State, Callable, type(None)])
    def _set_onClose(self, value):
        self._onClose = value

    def _get_onClose(self):
        return self._onClose.value if isinstance(self._onClose, State) else self._onClose

    @_validate_param(file_path="qtmui.material.dialog", param_name="PaperComponent", supported_signatures=Union[State, str, type(None)])
    def _set_PaperComponent(self, value):
        self._PaperComponent = value

    def _get_PaperComponent(self):
        return self._PaperComponent.value if isinstance(self._PaperComponent, State) else self._PaperComponent

    @_validate_param(file_path="qtmui.material.dialog", param_name="PaperProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_PaperProps(self, value):
        self._PaperProps = value

    def _get_PaperProps(self):
        return self._PaperProps.value if isinstance(self._PaperProps, State) else self._PaperProps

    @_validate_param(file_path="qtmui.material.dialog", param_name="scroll", supported_signatures=Union[State, str], valid_values=VALID_SCROLL)
    def _set_scroll(self, value):
        self._scroll = value

    def _get_scroll(self):
        return self._scroll.value if isinstance(self._scroll, State) else self._scroll

    @_validate_param(file_path="qtmui.material.dialog", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.dialog", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.dialog", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.dialog", param_name="TransitionComponent", supported_signatures=Union[State, str, type(None)])
    def _set_TransitionComponent(self, value):
        self._TransitionComponent = value

    def _get_TransitionComponent(self):
        return self._TransitionComponent.value if isinstance(self._TransitionComponent, State) else self._TransitionComponent

    # @_validate_param(file_path="qtmui.material.dialog", param_name="transitionDuration", supported_signatures=Union[State, int, Dict[str, int], type(None)])
    def _set_transitionDuration(self, value):
        if isinstance(value, dict) and not all(k in ["appear", "enter", "exit"] for k in value):
            raise ValueError("transitionDuration dict must contain 'appear', 'enter', or 'exit' keys")
        self._transitionDuration = value

    def _get_transitionDuration(self):
        return self._transitionDuration.value if isinstance(self._transitionDuration, State) else self._transitionDuration

    @_validate_param(file_path="qtmui.material.dialog", param_name="TransitionProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_TransitionProps(self, value):
        self._TransitionProps = value

    def _get_TransitionProps(self):
        return self._TransitionProps.value if isinstance(self._TransitionProps, State) else self._TransitionProps

    @_validate_param(file_path="qtmui.material.dialog", param_name="parent", supported_signatures=Union[QWidget, type(None)])
    def _set_parent(self, value):
        self._parent = value

    def _get_parent(self):
        return self._parent

    @_validate_param(file_path="qtmui.material.dialog", param_name="borderRadius", supported_signatures=Union[State, int])
    def _set_borderRadius(self, value):
        self._borderRadius = value

    def _get_borderRadius(self):
        return self._borderRadius.value if isinstance(self._borderRadius, State) else self._borderRadius

    @_validate_param(file_path="qtmui.material.dialog", param_name="onButtonOkClicked", supported_signatures=Union[State, Callable, type(None)])
    def _set_onButtonOkClicked(self, value):
        self._onButtonOkClicked = value

    def _get_onButtonOkClicked(self):
        return self._onButtonOkClicked.value if isinstance(self._onButtonOkClicked, State) else self._onButtonOkClicked

    @_validate_param(file_path="qtmui.material.dialog", param_name="width", supported_signatures=Union[State, int, type(None)])
    def _set_width(self, value):
        self._width = value

    def _get_width(self):
        return self._width.value if isinstance(self._width, State) else self._width

    @_validate_param(file_path="qtmui.material.dialog", param_name="height", supported_signatures=Union[State, int, type(None)])
    def _set_height(self, value):
        self._height = value

    def _get_height(self):
        return self._height.value if isinstance(self._height, State) else self._height

    @_validate_param(file_path="qtmui.material.dialog", param_name="title", supported_signatures=Union[State, QWidget, str, type(None)])
    def _set_title(self, value):
        self._title = value

    def _get_title(self):
        return self._title.value if isinstance(self._title, State) else self._title

    @_validate_param(file_path="qtmui.material.dialog", param_name="transition", supported_signatures=Union[State, str, type(None)])
    def _set_transition(self, value):
        self._transition = value

    def _get_transition(self):
        return self._transition.value if isinstance(self._transition, State) else self._transition

    def __init_ui(self):
        if isinstance(self._open, State):
            self._open.valueChanged.connect(self._set_visible)
            if self._open.value == True:
                self._set_visible(self._open.value)

        self.vBoxLayout = QVBoxLayout(self.widget)
        self._hide_widget()

        if self._fullScreen:
            self.layout().setContentsMargins(0,50,0,0)
            self.widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
            self._borderRadius = 0
        else:
            self.widget.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            if self._width:
                self.widget.setFixedWidth(self._width)
            elif self._maxWidth:
                self.widget.setFixedWidth(self._maxWidth)
            if self._height:
                self.widget.setFixedHeight(self._height)

        self.widget.setObjectName(str(uuid.uuid4()))

        self.setShadowEffect(60, (0, 10), QColor(0, 0, 0, 80))
        self.setMaskColor(QColor(0, 0, 0, 76))

        self.vBoxLayout.setSpacing(0)
        self.vBoxLayout.setAlignment(Qt.AlignTop)


        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def add_widget(self, widget):
        self.vBoxLayout.addWidget(widget)

    def add_widgets(self, widgets):
        if widgets:
            if not isinstance(widgets, list):
                raise TypeError("widgets must be type (list)")
            for child in widgets:
                self.vBoxLayout.addWidget(child)

    def _hide_widget(self):
        if self._transition and self._transition == "fadeIn":
            self.hide_anim = QPropertyAnimation(self.widget, b"geometry")
            self.hide_anim.setDuration(5) 
            self.hide_anim.setStartValue(QRect(self.widget.x(), self.widget.y() + 2000, self.widget.width(), self.widget.height())) 
            self.hide_anim.setEndValue(QRect(self.widget.x(), self.widget.y() + 1500, self.widget.width(), self.widget.height())) 
            self.hide_anim.start()

    def _setupChildrenAndShow(self):
        """ fade in """
        if not self.vBoxLayout.count():
            children = []
            if isinstance(self._children, Callable):
                children = self._children()
            elif isinstance(self._children, list):
                children = self._children

            if isinstance(children, list):
                for child in children:
                    if child is not None:
                        self.vBoxLayout.addWidget(child)
                        
        # if self._transition and self._transition == "fadeIn":
        #     self.widget.show()
        #     self.widget.adjustSize()
        #     self.hide_anim = QPropertyAnimation(self.widget, b"geometry")
        #     self.hide_anim.setDuration(500) 
        #     self.hide_anim.setStartValue(QRect(self.widget.x(), self.widget.y() + 2000, self.widget.width(), self.widget.height())) 
        #     self.hide_anim.setEndValue(QRect(self.widget.x(), self.widget.y(), self.widget.width(), self.widget.height())) 
        #     # self.hide_anim.finished.connect(self.widget.show)
        #     self.hide_anim.start()

        if QApplication.instance().mainWindow:
            self.setParent(QApplication.instance().mainWindow)
            self.setGeometry(0, 0, QApplication.instance().mainWindow.width(), QApplication.instance().mainWindow.height())
            if not self.isVisible():
                self.show()
            if not self.widget.isVisible():
                self.widget.show()


    def showEvent(self, e):
        self.setParent(QApplication.instance().mainWindow)
        QTimer.singleShot(0, self._setupChildrenAndShow)
        # AnimManager.showEvent(self.widget, e)
        super().showEvent(e)
        
    def hideEvent(self, event):
        if self._onClose:
            self._onClose()
        return super().hideEvent(event)

    def _show_widget(self):
        self.widget.show()

    def mousePressEvent(self, event):
        if not self.widget.underMouse():
            self.hide()
        return super().mousePressEvent(event)

    def _set_visible(self, show):
        if show:
            self.show()
        elif show is not None:
            self.hide()

    def _accept(self):
        self.onButtonOkClicked()
        
    def _reject(self):
        self.close()

    def _set_stylesheet(self, component_styled=None):
        super()._set_stylesheet(component_styled)
        
        self.theme = useTheme()

        ownerState = {}

        if not component_styled:
            component_styled = self.theme.components


        PyDialog = component_styled["PyDialog"].get("styles")
        PyDialogPaperRootStyles = get_qss_style(PyDialog["paper"]["root"])
        PyDialogPaperFullScreenStyles = get_qss_style(PyDialog["paperFullScreen"])

        PyDialogPaperRootSlotFullScreenStyles = get_qss_style(PyDialog["paper"]["root"]["slots"]["fullScreen"])


        self.widget.setStyleSheet(f"""
            #{self.widget.objectName()} {{
                {PyDialogPaperRootStyles}
                {PyDialogPaperFullScreenStyles if self._fullScreen else ""}
                {PyDialogPaperRootSlotFullScreenStyles if self._fullScreen else ""}
            }}
            QScrollBar:horizontal {{
                border: none;
                background: transparent;
                height: 8px;
                margin: 0px 21px 0 21px;
                border-radius: 0px;
            }}

            QScrollBar::handle:horizontal {{
                background: {alpha(self.theme.palette.grey._500, 0.32)};
                min-width: 25px;
                border-radius: 2px;
                margin-bottom: 2px;
            }}

            QScrollBar::add-line:horizontal {{
                border: none;
                background: transparent;
                width: 20px;
                border-top-right-radius: 0px;
                border-bottom-right-radius: 0px;
                subcontrol-position: right;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:horizontal {{
                border: none;
                background: transparent;
                width: 20px;
                border-top-left-radius: 0px;
                border-bottom-left-radius: 0px;
                subcontrol-position: left;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:horizontal,
            QScrollBar::down-arrow:horizontal {{
                background: none;
            }}

            QScrollBar::add-page:horizontal,
            QScrollBar::sub-page:horizontal {{
                background: none;
            }}

            QScrollBar:vertical {{
                border: none;
                background: transparent;
                width: 8px;
                margin: 21px 0 21px 0;
                border-radius: 0px;
            }}

            QScrollBar::handle:vertical {{
                background: {alpha(self.theme.palette.grey._500, 0.32)};
                min-height: 25px;
                border-radius: 2px;
                margin-right: 2px;
            }}


            QScrollBar::add-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-bottom-left-radius: 0px; 
                border-bottom-right-radius: 0px;
                subcontrol-position: bottom;
                subcontrol-origin: margin;
            }}

            QScrollBar::sub-line:vertical {{
                border: none;
                background: transparent;
                height: 20px;
                border-top-left-radius: 0px;
                border-top-right-radius: 0px;
                subcontrol-position: top;
                subcontrol-origin: margin;
            }}

            QScrollBar::up-arrow:vertical,
            QScrollBar::down-arrow:vertical {{
                background: none;
            }}

            QScrollBar::add-page:vertical,
            QScrollBar::sub-page:vertical {{
                background: none;
            }}
        """)






from PySide6.QtWidgets import QWidget, QHBoxLayout
from PySide6.QtCore import QObject


class TableHead(QWidget):
    def __init__(
                self,
                children: object = None
                ):
        super().__init__()
        self._children = children

        self._init_ui()

    def _init_ui(self):
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        if self._children:
            for widget in self._children:
                self.layout().addWidget(widget)


# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and ..site_packages.qtcompat
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT QT CORE
# ///////////////////////////////////////////////////////////////
from PySide6.QtWidgets import QPushButton, QLabel, QGraphicsDropShadowEffect, QApplication
from PySide6.QtCore import Qt, QEvent, QRect, QPoint
from PySide6.QtGui import Qt, QPainter, QBrush, QColor, QPixmap


# PY TITLE BUTTON
# ///////////////////////////////////////////////////////////////
class PyTitleButton(QPushButton):
    def __init__(
        self,
        parent,
        tooltip_text = "",
        btn_id = None,
        width = 30,
        height = 30,
        radius = 8,
        bg_color = "rgb(249, 249, 249)",
        bg_color_hover = "rgb(242, 242, 242)",
        bg_color_pressed = "rgb(242, 242, 242)",
        icon_color = "#c3ccdf",
        icon_color_hover = "#dce1ec",
        icon_color_pressed = "#edf0f5",
        icon_color_active = "#f5f6f9",
        icon_path = "no_icon.svg",
        dark_one = "rgb(249, 249, 249)",
        context_color = "#568af2",
        text_foreground = "#8a95aa",
        is_active = False
    ):
        super().__init__()
        
        # SET DEFAULT PARAMETERS
        self.setFixedSize(width, height)
        self.setCursor(Qt.PointingHandCursor)
        self.setObjectName(btn_id)

        # PROPERTIES
        self._bgColor = bg_color
        self._bgColor_hover = bg_color_hover
        self._bgColor_pressed = bg_color_pressed        
        self._icon_color = icon_color
        self._icon_color_hover = icon_color_hover
        self._icon_color_pressed = icon_color_pressed
        self._icon_color_active = icon_color_active
        self._context_color = context_color
        self._top_margin = self.height() + 6
        self._is_active = is_active
        # Set Parameters
        self._set_bgColor = bg_color
        self._set_icon_path = icon_path
        self._set_icon_color = icon_color
        self._set_border_radius = radius
        # Parent
        self._parent = parent

        # TOOLTIP
        self._tooltip_text = tooltip_text
        self._tooltip = _ToolTip(
            tooltip_text,
            dark_one,
            context_color,
            text_foreground
        )
        self._tooltip.hide()

    # SET ACTIVE MENU
    # ///////////////////////////////////////////////////////////////
    def set_active(self, is_active):
        self._is_active = is_active
        self.repaint()

    # RETURN IF IS ACTIVE MENU
    # ///////////////////////////////////////////////////////////////
    def is_active(self):
        return self._is_active

    # PAINT EVENT
    # painting the button and the icon
    # ///////////////////////////////////////////////////////////////
    def paintEvent(self, event):
        # PAINTER
        paint = QPainter()
        paint.begin(self)
        paint.setRenderHint(QPainter.RenderHint.Antialiasing)
        
        if self._is_active:
            # BRUSH
            brush = QBrush(QColor(self._context_color))
        else:
            # BRUSH
            brush = QBrush(QColor(self._set_bgColor))

        # CREATE RECTANGLE
        rect = QRect(0, 0, self.width(), self.height())
        paint.setPen(Qt.NoPen)
        paint.setBrush(brush)
        paint.drawRoundedRect(
            rect, 
            self._set_border_radius, 
            self._set_border_radius
        )

        # DRAW ICONS
        self.icon_paint(paint, self._set_icon_path, rect)

        # END PAINTER
        paint.end()

    # CHANGE STYLES
    # Functions with custom styles
    # ///////////////////////////////////////////////////////////////
    def change_style(self, event):
        if event == QEvent.Enter:
            self._set_bgColor = self._bgColor_hover
            self._set_icon_color = self._icon_color_hover
            self.repaint()         
        elif event == QEvent.Leave:
            self._set_bgColor = self._bgColor
            self._set_icon_color = self._icon_color
            self.repaint()
        elif event == QEvent.MouseButtonPress:            
            self._set_bgColor = self._bgColor_pressed
            self._set_icon_color = self._icon_color_pressed
            self.repaint()
        elif event == QEvent.MouseButtonRelease:
            self._set_bgColor = self._bgColor_hover
            self._set_icon_color = self._icon_color_hover
            self.repaint()

    # MOUSE OVER
    # Event triggered when the mouse is over the BTN
    # ///////////////////////////////////////////////////////////////
    def enterEvent(self, event):
        self.change_style(QEvent.Enter)
        self.move_tooltip()
        self._tooltip.show()

    # MOUSE LEAVE
    # Event fired when the mouse leaves the BTN
    # ///////////////////////////////////////////////////////////////
    def leaveEvent(self, event):
        self.change_style(QEvent.Leave)
        self.move_tooltip()
        self._tooltip.hide()

    # MOUSE PRESS
    # Event triggered when the left button is pressed
    # ///////////////////////////////////////////////////////////////
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.change_style(QEvent.MouseButtonPress)
            # SET FOCUS
            self.setFocus()
            # EMIT SIGNAL
            return self.clicked.emit()

    # MOUSE RELEASED
    # Event triggered after the mouse button is released
    # ///////////////////////////////////////////////////////////////
    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.change_style(QEvent.MouseButtonRelease)
            # EMIT SIGNAL
            return self.released.emit()

    # DRAW ICON WITH COLORS
    # ///////////////////////////////////////////////////////////////
    def icon_paint(self, qp, image, rect):
        icon = QPixmap(image)
        painter = QPainter(icon)
        painter.setCompositionMode(QPainter.CompositionMode_SourceIn)
        if self._is_active:
            painter.fillRect(icon.rect(), self._icon_color_active)
        else:
            painter.fillRect(icon.rect(), self._set_icon_color)
        qp.drawPixmap(
            (rect.width() - icon.width()) / 2, 
            (rect.height() - icon.height()) / 2,
            icon
        )        
        painter.end()

    # SET ICON
    # ///////////////////////////////////////////////////////////////
    def set_icon(self, icon_path):
        self._set_icon_path = icon_path
        self.repaint()

    # MOVE TOOLTIP
    # ///////////////////////////////////////////////////////////////
    def move_tooltip(self):
        # GET MAIN WINDOW PARENT
        gp = self.mapToGlobal(QPoint(0, 0))

        # SET WIDGET TO GET POSTION
        # Return absolute position of widget inside app
        pos = self._parent.mapFromGlobal(gp)

        # FORMAT POSITION
        # Adjust tooltip position with offset
        pos_x = (pos.x() - self._tooltip.width()) + self.width() + 5
        pos_y = pos.y() + self._top_margin

        # SET POSITION TO WIDGET
        # Move tooltip position
        self._tooltip.move(pos_x, pos_y)

# TOOLTIP
# ///////////////////////////////////////////////////////////////
class _ToolTip(QLabel):
    # TOOLTIP / LABEL StyleSheet
    style_tooltip = """ 
    QLabel {{		
        background-color: {_dark_one};	
        color: {_text_foreground};
        padding-left: 10px;
        padding-right: 10px;
        border-radius: 17px;
        border: 0px solid transparent;
        border-right: 3px solid {_context_color};
        font: 800 9pt "Segoe UI";
    }}
    """
    def __init__(
        self,
        tooltip,
        dark_one,
        context_color,
        text_foreground
    ):
        QLabel.__init__(self)

        # LABEL SETUP
        style = self.style_tooltip.format(
            _dark_one = dark_one,
            _context_color = context_color,
            _text_foreground = text_foreground
        )
        self.setObjectName(u"label_tooltip")
        self.setStyleSheet(style)
        self.setMinimumHeight(34)
        self.setParent(QApplication.instance().mainWindow)
        self.setText(tooltip)
        self.adjustSize()

        # SET DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(30)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 80))
        self.setGraphicsEffect(self.shadow)



import asyncio
from functools import lru_cache
from typing import Callable, Optional, Union, Dict
from PySide6.QtWidgets import QPushButton
from PySide6.QtGui import QPainter, QPen, QColor
from PySide6.QtCore import Qt, Signal, QSize, QTimer

from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor
from ..system.color_manipulator import hex_string_to_qcolor
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.py_iconify import Iconify
from qtmui.hooks import State, useEffect
from qtmui.material.styles import useTheme
from qtmui.hooks import State

from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

COLORS = ['inherit', 'default', 'primary', 'secondary', 'info', 'success', 'warning', 'error']
SIZES = ['small', 'medium']

class Radio(QPushButton):
    """
    A radio button component, styled like Material-UI Radio.

    The `Radio` component allows users to select one option from a set. It integrates with the `qtmui`
    framework, retaining existing parameters (except `text` and `hightLight`), adding new parameters,
    and aligning with MUI Radio props. Inherits from `Button` to support `ButtonBase` props.

    Parameters
    ----------
    id : State or str, optional
        The id of the input element. Default is None.
        Can be a `State` object for dynamic updates.
    checked : State or bool, optional
        If True, the component is checked. Default is False.
        Can be a `State` object for dynamic updates.
    icon : State, QWidget, Callable, or None, optional
        The icon to display when unchecked. Default is None.
        Can be a `State` object for dynamic updates or a Callable returning a QWidget.
    checkedIcon : State, QWidget, Callable, or None, optional
        The icon to display when checked. Default is None.
        Can be a `State` object for dynamic updates or a Callable returning a QWidget.
    color : State or str, optional
        The color of the component ('default', 'primary', 'secondary', 'error', 'info', 'success', 'warning').
        Default is 'primary'.
        Can be a `State` object for dynamic updates.
    name : State or str, optional
        Name attribute of the input element. Default is None.
        Can be a `State` object for dynamic updates.
    onChange : State or Callable, optional
        Callback fired when the state changes. Default is None.
        Can be a `State` object for dynamic updates.
        Signature: (event: Any, checked: bool) -> None
    value : State or Any, optional
        The value of the component. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the component is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    disableRipple : State or bool, optional
        If True, the ripple effect is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    inputProps : State or dict, optional
        Attributes applied to the input element. Default is None.
        Can be a `State` object for dynamic updates.
    inputRef : State or Callable, optional
        Ref to the input element. Default is None.
        Can be a `State` object for dynamic updates.
    required : State or bool, optional
        If True, the input element is required. Default is False.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        The size of the component ('small', 'medium'). Default is 'medium'.
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for slots ({input, root}). Default is {}.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for slots ({input, root}). Default is {}.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `Button` class,
        supporting props of the `ButtonBase` component (e.g., disableElevation).

    Signals
    -------
    changed : Signal
        Emitted when the checked state or value changes.

    Notes
    -----
    - Existing parameters `text` and `hightLight` are removed; all other parameters are retained.
    - New parameters added to align with MUI: `classes`, `disabled`, `disableRipple`, `inputProps`,
      `inputRef`, `required`, `size`, `slotProps`, `slots`, `sx`.
    - Props of the `ButtonBase` component are supported via `**kwargs`.
    - MUI classes applied: `MuiRadio-root`, `Mui-checked`, `Mui-disabled`.
    - Integrates with `Button` for `ButtonBase` props and supports custom icons via `QWidget`.

    Demos:
    - Radio: https://qtmui.com/material-ui/qtmui-radio/

    API Reference:
    - Radio API: https://qtmui.com/material-ui/api/radio/
    """

    changed = Signal(object)

    def __init__(self,
                key: str = None,
                checked: bool = False,
                icon: str = None,
                checkedIcon: str = None,
                color: str = "default",
                disabled: Union[State, bool] = False,
                name: str = None,
                onChange: Callable = None,
                text: Optional[Union[str, State, Callable]] = None,
                value: object = None,
                size: Union[State, str] = "medium",
                sx: Optional[Union[State, Dict, Callable, str]] = None,
                *args, **kwargs
                ):
        super().__init__()
        self.setObjectName(str(id(self)))

        if sx:
            self._setSx(sx)
            
        self._kwargs = {
            **kwargs,
            "size": size,
            "color": color,
        }
            
        self._setKwargs(self._kwargs)

        self._key = key
        self._icon = icon
        
        self._color = color
        self._checked = checked
        self._checkedIcon = checkedIcon
        self._disabled = disabled
        self._onChange = onChange
        self._name = name
        self._value = value
        self._size = size
        self._sx = sx
        
        self._init_ui()
        
    def _get_checked(self):
        return self._checked.value if isinstance(self._checked, State) else self._checked
    
    def _init_ui(self):
        if self._disabled:
            self.setEnabled(False)
        else:
            self.setCursor(Qt.PointingHandCursor)
            
        self.setCheckable(True)
        self.clicked.connect(self._updateIcon)
        self.clicked.connect(self._onClicked)
        
        self._setCheckedState()
        
        self._setStyleSheet()

    def _onClicked(self):
        if self._onChange:
            self._onChange(self.isChecked())

    def _connectStates(self):
        if isinstance(self._checked, State):
            useEffect(
                self._setCheckedState,
                [self._checked]
            )
    
    def _setCheckedState(self, state=None):
        if state is None:
            state = self._get_checked()
        self.setChecked(state)
        self._updateIcon()
        
    
    def _updateIcon(self):
        self.theme = useTheme()
        
        if self.isChecked():
            if self._color == "inherit":
                color = self.theme.palette.text.primary
            elif self._color == "default":
                color = self.theme.palette.text.secondary
            elif self._color in COLORS:
                color = getattr(self.theme.palette, self._color).main
            else:
                color = self.theme.palette.text.secondary
                
            if isinstance(self._checkedIcon, Iconify):
                self._checkedIcon._color = color
                self.setIcon(self._checkedIcon.qIcon())
                print(self._checkedIcon._key)
            else:
                self.setIcon(Iconify(key="akar-icons:radio-fill", color=color).qIcon())
        else:
            if isinstance(self._icon, Iconify):
                self._icon._color = self.theme.palette.grey._500
                self.setIcon(self._icon.qIcon())
            else:
                self.setIcon(Iconify(key="ion:radio-button-off", color=self.theme.palette.grey._500).qIcon())
                
        MuiRadio_style_icon_size = self.theme.components["MuiRadio"]["styles"]["icon"](self._kwargs)["size"]
        self.setIconSize(QSize(MuiRadio_style_icon_size["width"], MuiRadio_style_icon_size["height"]))
                
    @classmethod
    def _setSx(cls, sx: dict = {}):
        cls.sxDict = sx
        
    @classmethod
    def _setKwargs(cls, kwargs: dict = {}):
        cls.ownerState = kwargs

    @classmethod
    @lru_cache(maxsize=128)
    def _getSxQss(cls, sxStr: str = "", className: str = "PyWidgetBase"):
        sx_qss = get_qss_style(cls.sxDict, class_name=className)
        return sx_qss

    @classmethod
    @lru_cache(maxsize=128)
    def _getStyleSheet(cls, objectName, _styledConfig: str, _size: str, _color: str, _theme_mode: str):
        theme = useTheme()

        MuiRadio_styles = theme.components[f"MuiRadio"].get("styles")
        MuiRadio_root_qss = get_qss_style(MuiRadio_styles["root"](cls.ownerState))
        MuiRadio_root_color_qss = get_qss_style(MuiRadio_styles["root"](cls.ownerState)[_color])
        MuiRadio_root_color_slot_hover_qss = get_qss_style(MuiRadio_styles["root"](cls.ownerState)[_color]["slots"]["hover"])
        
        _text_color = MuiRadio_styles["root"](cls.ownerState)[_color]["color"]
        _icon_color = MuiRadio_styles["icon"](cls.ownerState)["color"]

        _indicator_border_width = MuiRadio_styles["checkedIndicator"]["border-width"]
        _indicator_border_radius = MuiRadio_styles["checkedIndicator"]["border-radius"]
        _indicator_padding = MuiRadio_styles["checkedIndicator"]["padding"]

        stylesheet = f"""
            #{objectName} {{
                {MuiRadio_root_qss}
                {MuiRadio_root_color_qss}
            }}
            #{objectName}:hover {{
                {MuiRadio_root_color_slot_hover_qss}
            }}
        """
        
        # print('stylesheet____________________', stylesheet)

        return stylesheet

    def _setStyleSheet(self):
        
        stylesheet = ""
        if hasattr(self, "styledDict"):
            root = self.styledDict.get("MuiRadio", {}).get("styles", {}).get("root", None)(self._kwargs)
            if root:
                stylesheet = self._getStyleSheet(
                    self.objectName(),
                    _styledConfig=str(root),
                    _size=self._size,
                    _color=self._color,
                    _theme_mode=useTheme().palette.mode,
                )
        else:
            stylesheet = self._getStyleSheet(
                self.objectName(),
                _styledConfig="Radio",
                _size=self._size,
                _color=self._color,
                _theme_mode=useTheme().palette.mode,
            )
            
        sxQss = ""
        if self._sx:
            sxQss = self._getSxQss(sxStr=str(self._sx), className=f"#{self.objectName()}")
            
        if self._color == "inherit":
            self._color = "default"

        stylesheet = f"""
            {stylesheet}
            {sxQss}
        """
        
        self.setStyleSheet(stylesheet)
        


from functools import lru_cache
from typing import Any, Callable, List, Optional, Union, Dict
import uuid
from PySide6.QtWidgets import (
    QVBoxLayout, 
    QTabBar, 
    QWidget, 
    QHBoxLayout, 
    QLabel, 
    QSizePolicy, 
    QTabWidget,
    QWidget, 
    QSizePolicy, 
    QPushButton,
    QToolButton,
)

from PySide6.QtCore import Qt
from PySide6.QtGui import QIcon, QCursor

from ...material.system.color_manipulator import alpha

from qtmui.material.py_iconify import Iconify

from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from qtmui.material.styles.create_theme.create_palette import PaletteColor

from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from .tab import Tab
from qtmui.hooks import State
from ..widget_base import PyWidgetBase
from ..utils.validate_params import _validate_param

class Tabs(QTabWidget, PyWidgetBase):
    """
    A tabs component, styled like Material-UI Tabs.

    The `Tabs` component manages a group of tabs with scrollable or centered layouts, aligning with
    MUI Tabs props. Inherits from native component props.

    Parameters
    ----------
    justifyContent : State, str, or None, optional
        Alignment of tabs ('flex-start', 'flex-end', qtmui-specific). Default is 'flex-start'.
        Can be a `State` object for dynamic updates.
    orientation : State, str, or None, optional
        Orientation of the tab bar ('horizontal', 'vertical'). Default is 'horizontal'.
        Can be a `State` object for dynamic updates.
    value : State, Any, or None, optional
        Value of the selected tab. Default is None.
        Can be a `State` object for dynamic updates.
    onChange : State, Callable, or None, optional
        Callback fired when the selected tab changes. Default is None.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, tabs use all available space (qtmui-specific). Default is True.
        Can be a `State` object for dynamic updates.
    fixedHeight : State, int, or None, optional
        Fixed height of the tab widget (qtmui-specific). Default is None.
        Can be a `State` object for dynamic updates.
    minWidth : State or bool, optional
        If True, sets minimum width (qtmui-specific). Default is True.
        Can be a `State` object for dynamic updates.
    children : State, List[QWidget], QWidget, or None, optional
        Content of the tabs (typically Tab components). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, List, Dict, Callable, or None, optional
        System prop for CSS overrides. Default is None.
        Can be a `State` object for dynamic updates.
    action : State, Any, or None, optional
        Callback for programmatic actions (updateIndicator, updateScrollButtons). Default is None.
        Can be a `State` object for dynamic updates.
    allowScrollButtonsMobile : State or bool, optional
        If True, scroll buttons are shown on mobile. Default is False.
        Can be a `State` object for dynamic updates.
    aria_label : State, str, or None, optional
        ARIA label for the Tabs. Default is None.
        Can be a `State` object for dynamic updates.
    aria_labelledby : State, str, or None, optional
        ARIA labelledby for the Tabs. Default is None.
        Can be a `State` object for dynamic updates.
    centered : State or bool, optional
        If True, tabs are centered. Default is False.
        Can be a `State` object for dynamic updates.
    classes : State or Dict, optional
        Override or extend styles. Default is None.
        Can be a `State` object for dynamic updates.
    component : State, str, or None, optional
        Component for the root node. Default is None.
        Can be a `State` object for dynamic updates.
    indicatorColor : State, str, or None, optional
        Color of the tab indicator ('primary', 'secondary', or custom). Default is 'primary'.
        Can be a `State` object for dynamic updates.
    scrollButtons : State, str, or bool, optional
        Behavior of scroll buttons ('auto', True, False). Default is 'auto'.
        Can be a `State` object for dynamic updates.
    selectionFollowsFocus : State or bool, optional
        If True, tab selection follows focus. Default is False.
        Can be a `State` object for dynamic updates.
    slotProps : State or Dict, optional
        Props for each slot (indicator, scrollButtons, etc.). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or Dict, optional
        Components for each slot (indicator, scrollButtons, etc.). Default is None.
        Can be a `State` object for dynamic updates.
    TabIndicatorProps : State or Dict, optional
        Props for tab indicator (deprecated, use slotProps.indicator). Default is None.
        Can be a `State` object for dynamic updates.
    TabScrollButtonProps : State or Dict, optional
        Props for scroll buttons (deprecated, use slotProps.scrollButtons). Default is None.
        Can be a `State` object for dynamic updates.
    textColor : State, str, or None, optional
        Color of tab text ('inherit', 'primary', 'secondary'). Default is 'primary'.
        Can be a `State` object for dynamic updates.
    variant : State, str, or None, optional
        Display behavior ('fullWidth', 'scrollable', 'standard'). Default is 'standard'.
        Can be a `State` object for dynamic updates.
    visibleScrollbar : State or bool, optional
        If True, scrollbar is visible. Default is False.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to QTabWidget, supporting native props.

    Signals
    -------
    currentChanged : Signal(int)
        Emitted when the selected tab changes.

    Notes
    -----
    - `justifyContent`, `fixedHeight`, `minWidth` are qtmui-specific features.
    - `ScrollButtonComponent`, `TabIndicatorProps`, `TabScrollButtonProps` are deprecated; use `slotProps` and `slots`.
    - Supports dynamic updates via State objects.
    - MUI classes applied: `MuiTabs-root`.

    Demos:
    - Tabs: https://qtmui.com/material-ui/qtmui-tabs/

    API Reference:
    - Tabs API: https://qtmui.com/material-ui/api/tabs/
    """

    VALID_ORIENTATIONS = ['horizontal', 'vertical']
    VALID_COLORS = ['primary', 'secondary', 'inherit']
    VALID_VARIANTS = ['fullWidth', 'scrollable', 'standard']
    VALID_SCROLL_BUTTONS = ['auto', True, False]

    def __init__(
        self,
        justifyContent: Optional[Union[State, str, None]]="flex-start",
        orientation: Optional[Union[State, str, None]]="horizontal",
        value: Optional[Union[State, Any, None]]=None,
        onChange: Optional[Union[State, Callable, None]]=None,
        fullWidth: Union[State, bool]=True,
        fixedHeight: Optional[Union[State, int, None]]=None,
        minWidth: Union[State, bool]=True,
        children: Optional[Union[State, List[QWidget], QWidget, None]]=None,
        sx: Optional[Union[State, List, Dict, Callable, None]]=None,
        action: Optional[Union[State, Any, None]]=None,
        allowScrollButtonsMobile: Union[State, bool]=False,
        aria_label: Optional[Union[State, str, None]]=None,
        aria_labelledby: Optional[Union[State, str, None]]=None,
        centered: Union[State, bool]=False,
        classes: Optional[Union[State, Dict, None]]=None,
        component: Optional[Union[State, str, None]]=None,
        indicatorColor: Optional[Union[State, str, None]]="primary",
        scrollButtons: Union[State, str, bool]="auto",
        selectionFollowsFocus: Union[State, bool]=False,
        slotProps: Optional[Union[State, Dict, None]]=None,
        slots: Optional[Union[State, Dict, None]]=None,
        TabIndicatorProps: Optional[Union[State, Dict, None]]=None,
        TabScrollButtonProps: Optional[Union[State, Dict, None]]=None,
        textColor: Optional[Union[State, str, None]]="primary",
        variant: Optional[Union[State, str, None]]="standard",
        visibleScrollbar: Union[State, bool]=False,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(str(uuid.uuid4()))
        PyWidgetBase._setUpUi(self)
        self.theme = useTheme()
        self._widget_references = []

        # Set properties with validation
        self._set_justifyContent(justifyContent)
        self._set_orientation(orientation)
        self._set_value(value)
        self._set_onChange(onChange)
        self._set_fullWidth(fullWidth)
        self._set_fixedHeight(fixedHeight)
        self._set_minWidth(minWidth)
        self._set_children(children)
        self._set_sx(sx)
        self._set_action(action)
        self._set_allowScrollButtonsMobile(allowScrollButtonsMobile)
        self._set_aria_label(aria_label)
        self._set_aria_labelledby(aria_labelledby)
        self._set_centered(centered)
        self._set_classes(classes)
        self._set_component(component)
        self._set_indicatorColor(indicatorColor)
        self._set_scrollButtons(scrollButtons)
        self._set_selectionFollowsFocus(selectionFollowsFocus)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_TabIndicatorProps(TabIndicatorProps)
        self._set_TabScrollButtonProps(TabScrollButtonProps)
        self._set_textColor(textColor)
        self._set_variant(variant)
        self._set_visibleScrollbar(visibleScrollbar)

        self.keys = []
        self._init_ui()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.tabs", param_name="justifyContent", supported_signatures=Union[State, str, type(None)])
    def _set_justifyContent(self, value):
        self._justifyContent = value

    def _get_justifyContent(self):
        return self._justifyContent.value if isinstance(self._justifyContent, State) else self._justifyContent

    @_validate_param(file_path="qtmui.material.tabs", param_name="orientation", supported_signatures=Union[State, str, type(None)], valid_values=VALID_ORIENTATIONS)
    def _set_orientation(self, value):
        self._orientation = value

    def _get_orientation(self):
        orientation = self._orientation.value if isinstance(self._orientation, State) else self._orientation
        return orientation if orientation in self.VALID_ORIENTATIONS else 'horizontal'

    # @_validate_param(file_path="qtmui.material.tabs", param_name="value", supported_signatures=Union[State, Any, type(None)])
    def _set_value(self, value):
        self._value = value

    def _get_value(self):
        return self._value.value if isinstance(self._value, State) else self._value

    @_validate_param(file_path="qtmui.material.tabs", param_name="onChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onChange(self, value):
        self._onChange = value

    def _get_onChange(self):
        return self._onChange.value if isinstance(self._onChange, State) else self._onChange

    @_validate_param(file_path="qtmui.material.tabs", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        self._fullWidth = value

    def _get_fullWidth(self):
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(file_path="qtmui.material.tabs", param_name="fixedHeight", supported_signatures=Union[State, int, type(None)])
    def _set_fixedHeight(self, value):
        self._fixedHeight = value

    def _get_fixedHeight(self):
        return self._fixedHeight.value if isinstance(self._fixedHeight, State) else self._fixedHeight

    @_validate_param(file_path="qtmui.material.tabs", param_name="minWidth", supported_signatures=Union[State, bool])
    def _set_minWidth(self, value):
        self._minWidth = value

    def _get_minWidth(self):
        return self._minWidth.value if isinstance(self._minWidth, State) else self._minWidth

    # @_validate_param(file_path="qtmui.material.tabs", param_name="children", supported_signatures=Union[State, List[QWidget], QWidget, type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.tabs", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    # @_validate_param(file_path="qtmui.material.tabs", param_name="action", supported_signatures=Union[State, Any, type(None)])
    def _set_action(self, value):
        self._action = value

    def _get_action(self):
        return self._action.value if isinstance(self._action, State) else self._action

    @_validate_param(file_path="qtmui.material.tabs", param_name="allowScrollButtonsMobile", supported_signatures=Union[State, bool])
    def _set_allowScrollButtonsMobile(self, value):
        self._allowScrollButtonsMobile = value

    def _get_allowScrollButtonsMobile(self):
        return self._allowScrollButtonsMobile.value if isinstance(self._allowScrollButtonsMobile, State) else self._allowScrollButtonsMobile

    @_validate_param(file_path="qtmui.material.tabs", param_name="aria_label", supported_signatures=Union[State, str, type(None)])
    def _set_aria_label(self, value):
        self._aria_label = value

    def _get_aria_label(self):
        return self._aria_label.value if isinstance(self._aria_label, State) else self._aria_label

    @_validate_param(file_path="qtmui.material.tabs", param_name="aria_labelledby", supported_signatures=Union[State, str, type(None)])
    def _set_aria_labelledby(self, value):
        self._aria_labelledby = value

    def _get_aria_labelledby(self):
        return self._aria_labelledby.value if isinstance(self._aria_labelledby, State) else self._aria_labelledby

    @_validate_param(file_path="qtmui.material.tabs", param_name="centered", supported_signatures=Union[State, bool])
    def _set_centered(self, value):
        self._centered = value

    def _get_centered(self):
        return self._centered.value if isinstance(self._centered, State) else self._centered

    @_validate_param(file_path="qtmui.material.tabs", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.tabs", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.tabs", param_name="indicatorColor", supported_signatures=Union[State, str, type(None)], valid_values=VALID_COLORS)
    def _set_indicatorColor(self, value):
        self._indicatorColor = value

    def _get_indicatorColor(self):
        color = self._indicatorColor.value if isinstance(self._indicatorColor, State) else self._indicatorColor
        return color if color in self.VALID_COLORS or isinstance(color, str) else 'primary'

    @_validate_param(file_path="qtmui.material.tabs", param_name="scrollButtons", supported_signatures=Union[State, str, bool], valid_values=VALID_SCROLL_BUTTONS)
    def _set_scrollButtons(self, value):
        self._scrollButtons = value

    def _get_scrollButtons(self):
        scroll = self._scrollButtons.value if isinstance(self._scrollButtons, State) else self._scrollButtons
        return scroll if scroll in self.VALID_SCROLL_BUTTONS else 'auto'

    @_validate_param(file_path="qtmui.material.tabs", param_name="selectionFollowsFocus", supported_signatures=Union[State, bool])
    def _set_selectionFollowsFocus(self, value):
        self._selectionFollowsFocus = value

    def _get_selectionFollowsFocus(self):
        return self._selectionFollowsFocus.value if isinstance(self._selectionFollowsFocus, State) else self._selectionFollowsFocus

    @_validate_param(file_path="qtmui.material.tabs", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.tabs", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.tabs", param_name="TabIndicatorProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_TabIndicatorProps(self, value):
        self._TabIndicatorProps = value

    def _get_TabIndicatorProps(self):
        return self._TabIndicatorProps.value if isinstance(self._TabIndicatorProps, State) else self._TabIndicatorProps

    @_validate_param(file_path="qtmui.material.tabs", param_name="TabScrollButtonProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_TabScrollButtonProps(self, value):
        self._TabScrollButtonProps = value

    def _get_TabScrollButtonProps(self):
        return self._TabScrollButtonProps.value if isinstance(self._TabScrollButtonProps, State) else self._TabScrollButtonProps

    @_validate_param(file_path="qtmui.material.tabs", param_name="textColor", supported_signatures=Union[State, str, type(None)], valid_values=VALID_COLORS)
    def _set_textColor(self, value):
        self._textColor = value

    def _get_textColor(self):
        color = self._textColor.value if isinstance(self._textColor, State) else self._textColor
        return color if color in self.VALID_COLORS else 'primary'

    @_validate_param(file_path="qtmui.material.tabs", param_name="variant", supported_signatures=Union[State, str, type(None)], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        self._variant = value

    def _get_variant(self):
        variant = self._variant.value if isinstance(self._variant, State) else self._variant
        return variant if variant in self.VALID_VARIANTS else 'standard'

    @_validate_param(file_path="qtmui.material.tabs", param_name="visibleScrollbar", supported_signatures=Union[State, bool])
    def _set_visibleScrollbar(self, value):
        self._visibleScrollbar = value

    def _get_visibleScrollbar(self):
        return self._visibleScrollbar.value if isinstance(self._visibleScrollbar, State) else self._visibleScrollbar


    def _init_ui(self):

        theme = useTheme()
        
        self.custom_widgets = {}
        
        self.hasIcon = False

        if self._fixedHeight:
            self.setFixedHeight(self._fixedHeight)

        # self.initUI()

        # self.setTabBar(TabBar())

        if self._minWidth:
            self.setMinimumWidth(self._minWidth)

        if self._orientation == "vertical":
            self.setTabPosition(QTabWidget.West)

        if self._justifyContent == "flex-end":
            self.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
            self._children.reverse()


        # Đặt tab bar vào một layout tùy chỉnh

        self._padding_for_only_icon = ""
        self._margin_left_for_icon = ""

        if isinstance(self._children, list) and len(self._children) > 0:
            for index, tab in enumerate(self._children, 0):
                if isinstance(tab, Tab):
                    if isinstance(tab._icon, Iconify) and tab._label:
                        self.hasIcon = True
                        if tab._iconPosition in ["end", "bottom", "top"]:
                            self._padding_for_only_icon = "padding-left: -20px;"
                            self.tabBar().setProperty("hasIconAndLabel", "true")
                            self.addTab(tab, self._getTranslatedText(tab._label))
                            self.set_tab_with_icon_at_position(
                                index, 
                                tab._label, 
                                tab._icon() if isinstance(tab._icon, Callable) else tab._icon, 
                                tab._iconPosition
                            )

                        else:
                            self.addTab(tab, tab._icon() if isinstance(tab._icon, Callable) else tab._icon, tab._label)
                    elif tab._label:
                        self.addTab(tab, self._getTranslatedText(tab._label))
                    elif isinstance(tab._icon, Iconify):
                        self._padding_for_only_icon = "padding-left: 24px;"
                        self.tabBar().setProperty("hasIcon", "true")
                        tab._icon._color = theme.palette.text.secondary
                        self.addTab(tab, tab._icon.qIcon(), "")

        # self.set_cursor_pointer_to_tabs()
        self.tabBar().setCursor(QCursor(Qt.PointingHandCursor))

        self.currentChanged.connect(self.on_tab_changed)

        if self._onChange:
            self.currentChanged.connect(lambda index: self._onChange(self.currentWidget()._key))

        if self.currentWidget():
            self.currentWidget()._init_ui()

        if self._value:
            if isinstance(self._value, State):
                self._value.valueChanged.connect(self.set_current_tab_by_key)
                self.set_current_tab_by_key(self._value.value)
            if isinstance(self._value, str):
                self.set_current_tab_by_key(self._value)

        self._update_stylesheet()
        self.theme.state.valueChanged.connect(self._update_stylesheet)
        self.destroyed.connect(self._on_destroyed)
        i18n.langChanged.connect(self.retranslateUi)



    def _update_stylesheet(self):
        """Slot to set the stylesheet."""
        self._set_stylesheet()

    async def _async_update_stylesheet(self):
        """Slot to set the stylesheet."""
        self._set_stylesheet()

    @classmethod
    @lru_cache(maxsize=128)
    def _get_stylesheet(cls, _theme_mode: str):
        
        theme = useTheme()
        # self._text_color_disabled = theme.palette.text.disabled # opacity
        # self._text_color_disabled = alpha(theme.palette.primary.main, theme.palette.action.disabledOpacity) # opacity
        # self._text_color_primary = theme.palette.text.primary # opacity
        # self._text_color_secondary = theme.palette.text.secondary # opacity
        
        PyTabWidget = theme.components[f"PyTabWidget"].get("styles")
        PyTabWidget_root_qss = get_qss_style(PyTabWidget["root"])
        PyTabWidget_pane_qss = get_qss_style(PyTabWidget["pane"])
        PyTabWidget_label_qss = get_qss_style(PyTabWidget["label"])
        PyTabWidget_label_slot_hover_qss = get_qss_style(PyTabWidget["label"]["slots"]["hover"])

        PyTabBar = theme.components[f"PyTabBar"].get("styles")
        PyTabBar_root = PyTabBar["root"]
        PyTabBar_tab = PyTabBar["tab"]
        PyTabBar_root_qss = get_qss_style(PyTabBar_root)
        PyTabBar_tab_qss = get_qss_style(PyTabBar_tab)
        PyTabBar_tab_slot_selected_qss = get_qss_style(PyTabBar_tab["slots"]["selected"])
        PyTabBar_tab_slot_notSelected_qss = get_qss_style(PyTabBar_tab["slots"]["notSelected"])
        PyTabBar_tab_prop_hasIcon_qss = get_qss_style(PyTabBar_tab["props"]["hasIcon"])
        PyTabBar_tab_prop_hasIconAndLabel_qss = get_qss_style(PyTabBar_tab["props"]["hasIconAndLabel"])

        # Apply the final styles
        stylesheet = f"""
            QTabWidget{{
                {PyTabWidget_root_qss}
            }}
            QTabWidget::pane{{
                {PyTabWidget_pane_qss}
            }}

            QTabBar {{
                {PyTabBar_root_qss}

            }}
            QTabBar::tab {{
                {PyTabBar_tab_qss}
            }}

            QTabBar::tab:selected {{
                {PyTabBar_tab_slot_selected_qss}
            }}

            QTabBar::tab:!selected {{
                {PyTabBar_tab_slot_notSelected_qss}
            }}

            QTabBar::tab[hasIcon=true] {{
                {PyTabBar_tab_prop_hasIcon_qss}
            }}
            QTabBar::tab[hasIconAndLabel=true] {{
                {PyTabBar_tab_prop_hasIconAndLabel_qss}
            }}

            QLabel {{
                {PyTabWidget_label_qss}
            }}

            QLabel::hover {{
                {PyTabWidget_label_slot_hover_qss}
            }}
            
        """

        return stylesheet

    def _set_stylesheet(self, component_styled=None):
        _theme_mode = useTheme().palette.mode
        stylesheet = self._get_stylesheet(_theme_mode)
        sx_qss = ""

        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(stylesheet + sx_qss)


    def set_current_tab_by_key(self, key):
        for i in range(self.count()):
            tab = self.widget(i)
            if getattr(tab, '_key', None) == key:
                self.setCurrentIndex(i)
                return True  # Đã tìm thấy và thiết lập tab
        return False  # Không tìm thấy tab có _key khớ

    def retranslateUi(self):
       # Dùng phương thức count() để lấy số lượng tab và cập nhật tên của tất cả các tab
        for i in range(self.count()):
            tabBar = self.widget(i)  # Lấy ra tabText hiện tại
            if self.hasIcon:
                self.setTabText(i, "")  # Cập nhật tên tab mới với tabText cũ
            else:
                self.setTabText(i, self._getTranslatedText(tabBar._label))  # Cập nhật tên tab mới với tabText cũ
                
            # Lấy custom widget theo index
            try:
                label = self.custom_widgets[i].findChild(QLabel, f"label_text_{i}")
                if label:
                    label.setText(self._getTranslatedText(tabBar._label))
            except Exception as e:
                pass
                

    def set_tab_with_icon_at_position(self, index, title, icon, iconPosition="top"):
        # Tạo widget chứa text và icon
        custom_widget = QWidget(self)
        
        # Xử lý vị trí icon
        if iconPosition in ["top", "bottom"]:
            layout = QVBoxLayout(custom_widget)
        else:
            layout = QHBoxLayout(custom_widget)

        # QLabel cho text
        label_text = QLabel(self._getTranslatedText(title))
        label_text.setObjectName(f'label_text_{index}')
        
        # QLabel cho icon
        if isinstance(icon, QIcon):
            label_icon = QLabel()
            label_icon.setAlignment((Qt.AlignVCenter | Qt.AlignHCenter))
            label_icon.setPixmap(icon.pixmap(20, 20))  # Điều chỉnh kích thước icon
        elif isinstance(icon, Iconify):
            label_icon = QToolButton()
            label_icon.setIcon(icon.qIcon())  # Điều chỉnh kích thước icon
        else:
            label_icon = icon

        # Thêm icon và text vào layout theo vị trí icon
        if iconPosition == "top":
            layout.addWidget(label_icon)
            layout.addWidget(label_text)
            layout.setAlignment(Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter)
        elif iconPosition == "bottom":
            layout.addWidget(label_text)
            layout.addWidget(label_icon)
            layout.setAlignment(Qt.AlignmentFlag.AlignCenter | Qt.AlignmentFlag.AlignVCenter)
        elif iconPosition == "start":  # Đổi 'left' thành 'start'
            layout.addWidget(label_icon)
            layout.addWidget(label_text)
            layout.setAlignment(Qt.AlignCenter)
        elif iconPosition == "end":  # Đổi 'right' thành 'end'
            layout.addWidget(label_text)
            layout.addWidget(label_icon)
            layout.setAlignment(Qt.AlignCenter)

        # Căn chỉnh nội dung
        
        layout.setContentsMargins(0, 0, 0, 0)

        # Điều chỉnh kích thước cho custom_widget
        custom_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)

        # Điều chỉnh kích thước tối thiểu cho custom_widget để phù hợp với nội dung
        custom_widget.setMinimumSize(custom_widget.sizeHint().width() + 24, custom_widget.sizeHint().height())

        # Đặt widget vào tab
        self.setTabText(index, "")
        self.setTabIcon(index, QIcon())  # Xóa icon gốc nếu có
        self.setTabEnabled(index, True)

        self.custom_widgets[index] = custom_widget
        # Thêm widget tùy chỉnh vào tab
        self.tabBar().setTabButton(index, QTabBar.RightSide, custom_widget)


    def set_cursor_pointer_to_tabs(self):
        tab_bar = self.tabBar()  # Lấy QTabBar của QTabWidget
        for i in range(tab_bar.count()):
            tab_bar.setCursor(i, QCursor(Qt.PointingHandCursor))  # Đặt cursor pointer cho từng tab


    def on_tab_changed(self, index):
        current_tab = self.widget(index)
        current_tab._init_ui()

    def disable_pointer_events(self):
        self.setCursor(Qt.BlankCursor)

    def paintEvent(self, arg__1):
        PyWidgetBase.paintEvent(self, arg__1)
        return super().paintEvent(arg__1)

    def resizeEvent(self, event):
        PyWidgetBase.resizeEvent(self, event)
        return super().resizeEvent(event)




import asyncio
import uuid
from typing import Optional, Callable, Any, Dict, Union

from PySide6.QtWidgets import QLineEdit, QHBoxLayout, QGroupBox
from PySide6.QtCore import Signal, Qt, QTimer
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.hooks import State

from .py_input import MuiInput

class MuiOutlinedInput(MuiInput):

    def __init__(
        self,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)

    
    def _set_stylesheet(self):
        super()._set_stylesheet()
        if not hasattr(self, "theme"):
            self.theme = useTheme()

        MuiOutlinedInput = self.theme.components["MuiOutlinedInput"].get("styles")
        MuiOutlinedInput_root = MuiOutlinedInput.get("root")
        MuiOutlinedInput_root_qss = get_qss_style(MuiOutlinedInput_root)
        MuiOutlinedInput_root_slot_hovered_qss = get_qss_style(MuiOutlinedInput_root["slots"]["hover"])
        MuiOutlinedInput_root_slot_focused_qss = get_qss_style(MuiOutlinedInput_root["slots"]["focus"])
        
        MuiOutlinedInput_inputField_root = MuiOutlinedInput.get("inputField")["root"]
        MuiOutlinedInput_inputField_root_qss = get_qss_style(MuiOutlinedInput_inputField_root)

        MuiOutlinedInput_root_prop_hasStartAdornment_qss = get_qss_style(MuiOutlinedInput_inputField_root["props"]["hasStartAdornment"])


        self.setProperty("variant", "outlined")


        self.setStyleSheet(
            self.styleSheet() + 
            f"""
                #{self.objectName()}[variant=outlined] {{
                    {MuiOutlinedInput_root_qss}
                }}
                #{self.objectName()}[variant=outlined]  QLineEdit, QPlainTextEdit, QTextEdit {{
                    {MuiOutlinedInput_inputField_root_qss}
                }}
                #{self.objectName()}[outlinedHasStartAdornment=true]  QLineEdit, QPlainTextEdit, QTextEdit {{
                    {MuiOutlinedInput_root_prop_hasStartAdornment_qss}
                }}
                #{self.objectName()}[hovered=true] {{
                    {MuiOutlinedInput_root_slot_hovered_qss}
                }}
                #{self.objectName()}[focused=true] {{
                    {MuiOutlinedInput_root_slot_focused_qss}
                }}
            """
        )


    def _set_prop_has_start_adornment(self, state:bool):
        self.setProperty("outlinedHasStartAdornment", state)
        self._set_stylesheet()

        



# coding:utf-8
from __future__ import annotations
from enum import Enum
from typing import Union, Literal
import warnings
from typing import TYPE_CHECKING

from PySide6.QtXml import QDomDocument
from PySide6.QtCore import QRectF, Qt, QFile, QObject, QRect,QSize
from PySide6.QtGui import QIcon, QIconEngine, QColor, QPixmap, QImage, QPainter, QAction
from PySide6.QtSvg import QSvgRenderer
from PySide6.QtWidgets import QApplication

import random, re


try:
    from pyconify import svg_path
except ModuleNotFoundError:  # pragma: no cover
    raise ModuleNotFoundError(
        "pyconify is required to use Iconify. "
        "Please install it with `pip install pyconify` or use the "
        "`pip install superqt[iconify]` extra."
    ) from None

if TYPE_CHECKING:
    Flip = Literal["horizontal", "vertical", "horizontal,vertical"]
    Rotation = Literal["90", "180", "270", 90, 180, 270, "-90", 1, 2, 3]


class Iconify(QIcon):
    """QIcon backed by an iconify icon.

    Iconify includes 150,000+ icons from most major icon sets including Bootstrap,
    FontAwesome, Material Design, and many more.

    Search availble icons at https://icon-sets.iconify.design
    Once you find one you like, use the key in the format `"prefix:name"` to create an
    icon:  `Iconify("bi:bell")`.

    This class is a thin wrapper around the
    [pyconify](https://github.com/pyapp-kit/pyconify) `svg_path` function. It pulls SVGs
    from iconify, creates a temporary SVG file and uses it as the source for a QIcon.
    SVGs are cached to disk, and persist across sessions (until `pyconify.clear_cache()`
    is called).

    Parameters are the same as `Iconify.addKey`, which can be used to add
    additional icons for various modes and states to the same QIcon.

    Parameters
    ----------
    *key: str
        Icon set prefix and name. May be passed as a single string in the format
        `"prefix:name"` or as two separate strings: `'prefix', 'name'`.
    color : str, optional
        Icon color. If not provided, the icon will appear black (the icon fill color
        will be set to the string "currentColor").
    flip : str, optional
        Flip icon.  Must be one of "horizontal", "vertical", "horizontal,vertical"
    rotate : str | int, optional
        Rotate icon. Must be one of 0, 90, 180, 270,
        or 0, 1, 2, 3 (equivalent to 0, 90, 180, 270, respectively)
    dir : str, optional
        If 'dir' is not None, the file will be created in that directory, otherwise a
        default
        [directory](https://docs.python.org/3/library/tempfile.html#tempfile.mkstemp) is
        used.

    Examples
    --------
    >>> from PySide6.QtWidgets import QPushButton
    >>> from superqt import Iconify
    >>> btn = QPushButton()
    >>> icon = Iconify("bi:alarm-fill", color="red", rotate=90)
    >>> btn.setIcon(icon)
    """

    def __init__(
        self,
        *key: str,
        color: str | None = None,
        flip: Flip | None = None,
        rotate: Rotation | None = None,
        dir: str | None = None,
        size = None,
        mode= QIcon.Mode.Normal,
        state = QIcon.State.Off
    ):
        super().__init__()
        if key:
            self.addKey(*key, 
                        color=color, 
                        flip=flip, 
                        rotate=rotate, 
                        dir=dir,
                        size = size,
                        mode= mode,
                        state = state)

    def addKey(
        self,
        *key: str,
        color: str | None = None,
        flip: Flip | None = None,
        rotate: Rotation | None = None,
        dir: str | None = None,
        size: QSize | None = None,
        mode: QIcon.Mode = QIcon.Mode.Normal,
        state: QIcon.State = QIcon.State.Off,
    ) -> Iconify:
        """Add an icon to this QIcon.

        This is a variant of `QIcon.addFile` that uses an iconify icon keys and
        arguments instead of a file path.

        Parameters
        ----------
        *key: str
            Icon set prefix and name. May be passed as a single string in the format
            `"prefix:name"` or as two separate strings: `'prefix', 'name'`.
        color : str, optional
            Icon color. If not provided, the icon will appear black (the icon fill color
            will be set to the string "currentColor").
        flip : str, optional
            Flip icon.  Must be one of "horizontal", "vertical", "horizontal,vertical"
        rotate : str | int, optional
            Rotate icon. Must be one of 0, 90, 180, 270, or 0, 1, 2, 3 (equivalent to 0,
            90, 180, 270, respectively)
        dir : str, optional
            If 'dir' is not None, the file will be created in that directory, otherwise
            a default
            [directory](https://docs.python.org/3/library/tempfile.html#tempfile.mkstemp)
            is used.
        size : QSize, optional
            Size specified for the icon, passed to `QIcon.addFile`.
        mode : QIcon.Mode, optional
            Mode specified for the icon, passed to `QIcon.addFile`.
        state : QIcon.State, optional
            State specified for the icon, passed to `QIcon.addFile`.

        Returns
        -------
        Iconify
            This Iconify instance, for chaining.
        """
        try:
            path = svg_path(*key, color=color, flip=flip, rotate=rotate, dir=dir)
        except OSError as e:
            warnings.warn(
                f"Error fetching icon: {e}.\nIcon {key} not cached. Using fallback.",
                stacklevel=2,
            )
            self._draw_text_fallback(key)
        else:
            self.addFile(str(path), size or QSize(), mode, state)

        return self

    def _draw_text_fallback(self, key: tuple[str, ...]) -> None:
        if style := QApplication.style():
            pixmap = style.standardPixmap(style.StandardPixmap.SP_MessageBoxQuestion)
        else:
            pixmap = QPixmap(18, 18)
            pixmap.fill(Qt.GlobalColor.transparent)
            painter = QPainter(pixmap)
            painter.drawText(pixmap.rect(), Qt.AlignmentFlag.AlignCenter, "?")
            painter.end()

        self.addPixmap(pixmap)


def writeSvg(iconPath: str, indexes=None, **attributes):
    """ write svg with specified attributes

    Parameters
    ----------
    iconPath: str
        svg icon path

    indexes: List[int]
        the path to be filled

    **attributes:
        the attributes of path

    Returns
    -------
    svg: str
        svg code
    """
    if not iconPath.lower().endswith('.svg'):
        return ""

    f = QFile(iconPath)
    f.open(QFile.OpenModeFlag.ReadOnly)

    dom = QDomDocument()
    dom.setContent(f.readAll())

    f.close()

    # change the color of each path
    pathNodes = dom.elementsByTagName('path')
    indexes = range(pathNodes.length()) if not indexes else indexes
    for i in indexes:
        element = pathNodes.at(i).toElement()

        for k, v in attributes.items():
            element.setAttribute(k, v)

    return dom.toString()


def changeSvgFill(iconPath: str, indexes=None, **attributes):
    """ write svg with specified attributes
    Parameters
    ----------
    iconPath: str
        svg icon path

    indexes: List[int]
        the path to be filled

    **attributes:
        the attributes of path

    Returns
    -------
    svg: str
        svg code
    """
    if not iconPath.lower().endswith('.svg'):
        return ""
    f = QFile(iconPath)
    f.open(QFile.OpenModeFlag.ReadOnly)
    dom = QDomDocument()
    dom.setContent(f.readAll())
    f.close()
    _attributes = dom.attributes()
    nodeType = dom.nodeType()
    # tagName = dom.tagName()
    # text = dom.text()
    documentElement = dom.documentElement()

    # change the color of each path
    pathNodes = dom.elementsByTagName('path')
    indexes = range(pathNodes.length()) if not indexes else indexes
    for i in indexes:
        element = pathNodes.at(i).toElement()
        for k, v in attributes.items():
            element.setAttribute(k, v)
    return dom.toString()


def drawSvgIcon(icon, painter, rect):
    """ draw svg icon

    Parameters
    ----------
    icon: str | bytes | QByteArray
        the path or code of svg icon

    painter: QPainter
        painter
    rect: QRect | QRectF
        the rect to render icon
    """
    renderer = QSvgRenderer(icon)
    renderer.setAnimationEnabled(True)
    renderer.render(painter, QRectF(rect))


class SvgIconEngine(QIconEngine):
    """ Svg icon engine """

    def __init__(self, svg: str):
        super().__init__()
        self.svg = svg

    def paint(self, painter, rect, mode, state):
        drawSvgIcon(self.svg.encode(), painter, rect)

    def clone(self) -> QIconEngine:
        return SvgIconEngine(self.svg)

    def pixmap(self, size, mode, state):
        image = QImage(size, QImage.Format.Format_ARGB32)
        image.fill(Qt.GlobalColor.transparent)
        pixmap = QPixmap.fromImage(image, Qt.ImageConversionFlag.NoFormatConversion)

        painter = QPainter(pixmap)
        rect = QRect(0, 0, size.width(), size.height())
        self.paint(painter, rect, mode, state)
        return pixmap


class DrawSVG:
    def path(self) -> str:
        raise NotImplementedError
    def icon(self, path, color: QColor|str = None) -> QIcon:
        """ create a fluent icon
        Parameters
        ----------
        path: str
            path to the svg file
        color: QColor | Qt.GlobalColor | str
            icon color, only applicable to svg icon
        """
        try:
            iconify = Iconify("material-symbols:auto-stories-outline", color="red")
            print(iconify)
            return iconify
        except Exception as e:
            print(e)
        if not (path.endswith('.svg') and color):
            return QIcon(path)
        color = QColor(color).name()
        return QIcon(SvgIconEngine(changeSvgFill(path, {"fill":color})))

    def render(self, painter: QPainter, rect:QRectF|QRect, icon:str, indexes=None, **attributes):
        """ draw svg icon

        Parameters
        ----------
        painter: QPainter
            painter

        rect: QRect | QRectF
            the rect to render icon

        theme: Theme
            the theme of icon
            * `Theme.Light`: black icon
            * `Theme.DARK`: white icon
            * `Theme.AUTO`: icon color depends on `config.theme`

        indexes: List[int]
            the svg path to be modified

        **attributes:
            the attributes of modified path
        """
        if icon.endswith('.svg'):
            if attributes:
                _icon = changeSvgFill(icon, indexes, **attributes).encode()
            drawSvgIcon(_icon, painter, rect)
        else:
            _icon = QIcon(icon)
            rect = QRectF(rect).toRect()
            painter.drawPixmap(rect, _icon.pixmap(QRectF(rect).toRect().size()))

class IconPath(DrawSVG, Enum):
    """ Fluent icon """
    UP = "Up"
    ADD = "Add"
    BUS = "Bus"
    CAR = "Car"
    CUT = "Cut"
    IOT = "IOT"
    PIN = "Pin"
    TAG = "Tag"
    VPN = "VPN"
    CAFE = "Cafe"
    CHAT = "Chat"
    COPY = "Copy"
    CODE = "Code"
    DOWN = "Down"
    EDIT = "Edit"
    FLAG = "Flag"
    FONT = "Font"
    GAME = "Game"
    HELP = "Help"
    HIDE = "Hide"
    HOME = "Home"
    INFO = "Info"
    LEAF = "Leaf"
    LINK = "Link"
    MAIL = "Mail"
    MENU = "Menu"
    MUTE = "Mute"
    MORE = "More"
    MOVE = "Move"
    PLAY = "Play"
    SAVE = "Save"
    SEND = "Send"
    SYNC = "Sync"
    UNIT = "Unit"
    VIEW = "View"
    WIFI = "Wifi"
    ZOOM = "Zoom"
    ALBUM = "Album"
    BRUSH = "Brush"
    BROOM = "Broom"
    CLOSE = "Close"
    CLOUD = "Cloud"
    EMBED = "Embed"
    GLOBE = "Globe"
    HEART = "Heart"
    LABEL = "Label"
    MEDIA = "Media"
    MOVIE = "Movie"
    MUSIC = "Music"
    ROBOT = "Robot"
    PAUSE = "Pause"
    PASTE = "Paste"
    PHOTO = "Photo"
    PHONE = "Phone"
    PRINT = "Print"
    SHARE = "Share"
    TILES = "Tiles"
    UNPIN = "Unpin"
    VIDEO = "Video"
    TRAIN = "Train"
    ADD_TO  ="AddTo"
    ACCEPT = "Accept"
    CAMERA = "Camera"
    CANCEL = "Cancel"
    DELETE = "Delete"
    FOLDER = "Folder"
    FILTER = "Filter"
    MARKET = "Market"
    SCROLL = "Scroll"
    LAYOUT = "Layout"
    GITHUB = "GitHub"
    UPDATE = "Update"
    REMOVE = "Remove"
    RETURN = "Return"
    PEOPLE = "People"
    QRCODE = "QRCode"
    RINGER = "Ringer"
    ROTATE = "Rotate"
    SEARCH = "Search"
    VOLUME = "Volume"
    FRIGID  = "Frigid"
    SAVE_AS = "SaveAs"
    ZOOM_IN = "ZoomIn"
    CONNECT  ="Connect"
    HISTORY = "History"
    SETTING = "Setting"
    PALETTE = "Palette"
    MESSAGE = "Message"
    FIT_PAGE = "FitPage"
    ZOOM_OUT = "ZoomOut"
    AIRPLANE = "Airplane"
    ASTERISK = "Asterisk"
    CALORIES = "Calories"
    CALENDAR = "Calendar"
    FEEDBACK = "Feedback"
    LIBRARY = "BookShelf"
    MINIMIZE = "Minimize"
    CHECKBOX = "CheckBox"
    DOCUMENT = "Document"
    LANGUAGE = "Language"
    DOWNLOAD = "Download"
    QUESTION = "Question"
    SPEAKERS = "Speakers"
    DATE_TIME = "DateTime"
    FONT_SIZE = "FontSize"
    HOME_FILL = "HomeFill"
    PAGE_LEFT = "PageLeft"
    SAVE_COPY = "SaveCopy"
    SEND_FILL = "SendFill"
    SKIP_BACK = "SkipBack"
    SPEED_OFF = "SpeedOff"
    ALIGNMENT = "Alignment"
    BLUETOOTH = "Bluetooth"
    COMPLETED = "Completed"
    CONSTRACT = "Constract"
    HEADPHONE = "Headphone"
    MEGAPHONE = "Megaphone"
    PROJECTOR = "Projector"
    EDUCATION = "Education"
    LEFT_ARROW = "LeftArrow"
    ERASE_TOOL = "EraseTool"
    PAGE_RIGHT = "PageRight"
    PLAY_SOLID = "PlaySolid"
    BOOK_SHELF = "BookShelf"
    HIGHTLIGHT = "Highlight"
    FOLDER_ADD = "FolderAdd"
    PAUSE_BOLD = "PauseBold"
    PENCIL_INK = "PencilInk"
    PIE_SINGLE = "PieSingle"
    QUICK_NOTE = "QuickNote"
    SPEED_HIGH = "SpeedHigh"
    STOP_WATCH = "StopWatch"
    ZIP_FOLDER = "ZipFolder"
    BASKETBALL = "Basketball"
    BRIGHTNESS = "Brightness"
    DICTIONARY = "Dictionary"
    MICROPHONE = "Microphone"
    ARROW_DOWN = "ChevronDown"
    FULL_SCREEN = "FullScreen"
    MIX_VOLUMES = "MixVolumes"
    REMOVE_FROM = "RemoveFrom"
    RIGHT_ARROW = "RightArrow"
    QUIET_HOURS  ="QuietHours"
    FINGERPRINT = "Fingerprint"
    APPLICATION = "Application"
    CERTIFICATE = "Certificate"
    TRANSPARENT = "Transparent"
    IMAGE_EXPORT = "ImageExport"
    SPEED_MEDIUM = "SpeedMedium"
    LIBRARY_FILL = "LibraryFill"
    MUSIC_FOLDER = "MusicFolder"
    POWER_BUTTON = "PowerButton"
    SKIP_FORWARD = "SkipForward"
    CARE_UP_SOLID = "CareUpSolid"
    ACCEPT_MEDIUM = "AcceptMedium"
    CANCEL_MEDIUM = "CancelMedium"
    CHEVRON_RIGHT = "ChevronRight"
    CLIPPING_TOOL = "ClippingTool"
    SEARCH_MIRROR = "SearchMirror"
    SHOPPING_CART = "ShoppingCart"
    FONT_INCREASE = "FontIncrease"
    BACK_TO_WINDOW = "BackToWindow"
    COMMAND_PROMPT = "CommandPrompt"
    CLOUD_DOWNLOAD = "CloudDownload"
    DICTIONARY_ADD = "DictionaryAdd"
    CARE_DOWN_SOLID = "CareDownSolid"
    CARE_LEFT_SOLID = "CareLeftSolid"
    CLEAR_SELECTION = "ClearSelection"
    DEVELOPER_TOOLS = "DeveloperTools"
    BACKGROUND_FILL = "BackgroundColor"
    CARE_RIGHT_SOLID = "CareRightSolid"
    CHEVRON_DOWN_MED = "ChevronDownMed"
    CHEVRON_RIGHT_MED = "ChevronRightMed"
    EMOJI_TAB_SYMBOLS = "EmojiTabSymbols"
    EXPRESSIVE_INPUT_ENTRY = "ExpressiveInputEntry"
    def path(self):
        return f':/qfluentwidgets/images/icons/{self.value}.svg'

class Icon(QIcon):
    def __init__(self, icon_path: IconPath):
        super().__init__(icon_path.path())
        self.icon_path = icon_path


def paintIcon(painter: QPainter, rect:QRectF|QRect, opacity:int=1,**attributes):
    svg = QSvgRenderer()
    svgBytes = changeSvgFill("", **attributes)
    svg.load(svgBytes)
    svg.render(painter, rect)
    painter.setOpacity(opacity)
    svg.setAnimationEnabled(True)

def drawIcon(icon, painter, rect, state=QIcon.State.Off, **attributes):
    """ draw icon
    Parameters
    ----------
    icon: str | QIcon | FluentIconBaseBase
        the icon to be drawn
    painter: QPainter
        painter
    rect: QRect | QRectF
        the rect to render icon
    **attribute:
        the attribute of svg icon
    """
    if isinstance(icon, DrawSVG):
        icon.render(painter, rect, **attributes)
    elif isinstance(icon, Icon):
        icon.icon_path.render(painter, rect, **attributes)
    else:
        icon = QIcon(icon)
        icon.paint(painter, QRectF(rect).toRect(), Qt.AlignmentFlag.AlignCenter, state=state)



class QssProperties:
    border = "border" # borders
    borderTop = "border-top" # borders
    borderLeft = "border-left" # borders
    borderRight = "border-right" # borders
    borderBottom = "border-bottom" # borders
    borderColor = "border-color" # palette
    borderTopColor = "border-top-color" # palette
    borderRightColor = "border-right-color" # palette
    borderBottomColor = "border-bottom-color" # palette
    borderLeftColor = "border-left-color" # palette
    borderRadius = "border-radius" # shape

    # system/spacing props
    margin = "margin" # spacing" # theme.spacing(value)
    m = "margin" # spacing" # theme.spacing(value)
    mt = "margin-top" # spacing" # theme.spacing(value)
    mr = "margin-right" # spacing" # theme.spacing(value)
    mb = "margin-bottom" # spacing" # theme.spacing(value)
    ml = "margin-left" # spacing" # theme.spacing(value)
    mx = ["margin-left", "margin-right"] # spacing" # theme.spacing(value)
    my = ["margin-top", "margin-bottom"] # spacing" # theme.spacing(value)
    marginTop = "margin-top" # spacing" # theme.spacing(value)
    marginRight = "margin-right" # spacing" # theme.spacing(value)
    marginBottom = "margin-bottom" # spacing" # theme.spacing(value)
    marginLeft = "margin-left" # spacing" # theme.spacing(value)
    marginX = ["margin-left", "margin-right"] # spacing" # theme.spacing(value)
    marginY = ["margin-top", "margin-bottom"] # spacing" # theme.spacing(value
    
    padding = "padding" # spacing" # theme.spacing(value)
    p = "padding" # spacing" # theme.spacing(value)
    pt = "padding-top" # spacing" # theme.spacing(value)
    pr = "padding-right" # spacing" # theme.spacing(value)
    pb = "padding-bottom" # spacing" # theme.spacing(value)
    pl = "padding-left" # spacing" # theme.spacing(value)
    px = ["padding-left", "padding-right"] # spacing" # theme.spacing(value)
    py = ["padding-top", "padding-bottom"] # spacing" # theme.spacing(value)
    paddingTop = "padding-top" # spacing" # theme.spacing(value)
    paddingRight = "padding-right" # spacing" # theme.spacing(value)
    paddingBottom = "padding-bottom" # spacing" # theme.spacing(value)
    paddingLeft = "padding-left" # spacing" # theme.spacing(value)
    paddingX = ["padding-left", "padding-right"] # spacing" # theme.spacing(value)
    paddingY = ["padding-top", "padding-bottom"] # spacing" # theme.spacing(value

    # system/typography props
    typography = "font-family, font-weight, font-size, line-height, letter-spacing, text-transform" # theme.typography[value]
    fontFamily = "font-family" # theme.typography[value]
    fontSize = "font-size" # theme.typography[value]
    fontStyle = "font-style" # theme.typography[value]
    fontWeight = "font-weight" # theme.typography[value]
    letterSpacing = "letter-spacing" # theme.typography[value]
    lineHeight = "line-height" # theme.typography[value]
    textAlign = "text-align" # textAlign
    textTransform = "text-transform" # textTransform

    boxShadow = "box-shadow" # boxShadow" # theme.shadows[value]
    displayPrint = "display" # displayPrint" # none
    display = "display" # displayRaw" # none
    alignContent = "align-content" # alignContent" # none
    alignItems = "align-items" # alignItems" # none
    alignSelf = "align-self" # alignSelf" # none
    flex = "flex" # flex" # none
    flexDirection = "flex-direction" # flexDirection" # none
    flexGrow = "flex-grow" # flexGrow" # none
    flexShrink = "flex-shrink" # flexShrink" # none
    flexWrap = "flex-wrap" # flexWrap" # none
    justifyContent = "justify-content" # justifyContent" # none
    order = "order" # order" # none
    opacity = "opacity" # order" # none
    gap = "gap" # gap" # theme.spacing(value)
    columnSpacing = "column-gap" # columnSpacing" # theme.spacing(value)
    rowSpacing = "row-gap" # rowSpacing" # theme.spacing(value)
    gridColumn = "grid-column" # gridColumn" # none
    gridRow = "grid-row" # gridRow" # none
    gridAutoFlow = "grid-auto-flow" # gridAutoFlow" # none
    gridAutoColumns = "grid-auto-columns" # gridAutoColumns" # none
    gridAutoRows = "grid-auto-rows" # gridAutoRows" # none
    gridTemplateColumns = "grid-template-columns" # gridTemplateColumns" # none
    gridTemplateRows = "grid-template-rows" # gridTemplateRows" # none
    gridTemplateAreas = "grid-template-areas" # gridTemplateAreas" # none
    gridArea = "grid-area" # gridArea" # none
    bgcolor = "background-color" # bgcolor" # theme.palette[value]
    bgImage = "background-image" # bgcolor" # theme.palette[value]
    backgroundImage = "background-image" # bgcolor" # theme.palette[value]
    color = "color" # color" # theme.palette[value]
    bottom = "bottom" # bottom" # none
    left = "left" # left" # none
    position = "position" # position" # none
    right = "right" # right" # none
    top = "top" # top" # none
    zIndex = "z-index" # zIndex" # theme.zIndex[value]
    height = "height" # height" # none
    maxHeight = "max-height" # maxHeight" # none
    maxWidth = "max-width" # maxWidth" # none
    minHeight = "min-height" # minHeight" # none
    minWidth = "min-width" # minWidth" # none
    width = "width" # width" # none
    boxSizing = "box-sizing" # boxSizing" # none

    subcontrolOrigin = 'subcontrol-origin'
    subcontrolPosition = 'subcontrol-osition'
    width = "width"



from typing import Callable
from PySide6.QtCore import QSize, QEvent, Signal, QRect
from PySide6.QtGui import QIcon, QColor, QBrush
from PySide6.QtWidgets import QStyledItemDelegate, QStyle

from .icon import icon_chrome_closed, icon_chrome_opened, icon_chrome_opening_closing, \
    icon_profile_edit, icon_profile_delete, icon_chrome_disabled, icon_firefox_closed, icon_firefox_opened, icon_firefox_opening_closing, icon_firefox_disabled

from ..utils.icon import icon_base64_to_pixmap


BUTTON_SIZE_X = 100  # Size of each button
BUTTON_SIZE_Y = 25  # Size of each button

class StyledItemDelegate(QStyledItemDelegate):
    
    """
    Stack
    Base container

    Args:
        column: index of column
        gap: int
        sx: str = QSS string
            {
                color: red;
                backroud: none;
            }
        alignItems: "space-around" | "space-between" | "space-evenly" | "stretch" | "center" | "end" | "flex-end" | "flex-start" | "start"
        justifyContent: "space-around" | "space-between" | "space-evenly" | "stretch" | "center" | "end" | "flex-end" | "flex-start" | "start"
        flexWrap: "wrap" | "no-wrap"

    Returns:
        new instance of PySyde6.QtWidgets.QFrame
    """

    mousePressedSignal = Signal(str)

    def __init__(self, 
                 parent=None,
                 column: int = None,
                 styledOption: object = None,
                 styledOptions: list = None,
                 onMousePressed: Callable = None
                 ):
        super(StyledItemDelegate, self).__init__(parent)
        if onMousePressed is not None:
            self.mousePressedSignal.connect(onMousePressed)
        self.column = column
        self.styledOption = styledOption
        self.styledOptions = styledOptions


    def split_rect(self, rect, num_divisions):
        # Tính toán chiều rộng của mỗi phần chia
        width_per_division = rect.width() // num_divisions
        
        # Tạo danh sách chứa các hình chữ nhật con
        sub_rects = []
        
        # Tạo các hình chữ nhật con
        for i in range(num_divisions):
            if i == 0:
                sub_rect = QRect((rect.left() + i * width_per_division) - 1, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
            if i == 1:
                sub_rect = QRect(rect.left() + i * width_per_division, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
            if i == 2:
                sub_rect = QRect((rect.left() + i * width_per_division) + 1, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
        return sub_rects

    def point_in_rects(self, point, rects):
        for i, rect in enumerate(rects):
            if rect.contains(point):
                return i
        return -1

    def editorEvent(self, event, model, option, index):
        sub_rects = self.split_rect(option.rect, 3)
        test_point = event.pos()

        # print('sub_rects___', sub_rects)

        # Kiểm tra xem test_point có nằm trong hình chữ nhật nào không
        result = self.point_in_rects(test_point, sub_rects)

        if option.state & QStyle.State_MouseOver:
            print('eoooooooooooooooooooooooooooeeeeeeeeeeee')
            option.backgroundBrush = QBrush(QColor(211, 211, 211))  # LightGray color

        if index.column() == 10 and event.type() == QEvent.MouseButtonRelease:
            if result != -1:
                if result == 0:
                    self.mousePressedSignal.emit(f"openclose_{index.row()}_{index.column()}")
                if result == 1:
                    self.mousePressedSignal.emit(f"edit_{index.row()}_{index.column()}")
                if result == 2:
                    self.mousePressedSignal.emit(f"delete_{index.row()}_{index.column()}")
                return True
            else:
                print("The point is not in any sub-rectangle.")

        return super().editorEvent(event, model, option, index)

    def paint(self, painter, option, index):
        if index.column() == self.column:  # Assuming the button column is the third column
            # buttons = [("Open", "Blac.png"), ("Edit", "Blac.png"), ("Delete", "Blac.png")]  # Define your button labels and icons
            items_count = len(self.styledOptions)
            total_button_width = items_count * BUTTON_SIZE_X
            width = option.rect.width()
            start_x = option.rect.x() + (width - total_button_width) // 2

            profile_state = index.data()

            for i, styledOption in enumerate(self.styledOptions):

                _rect = QRect(start_x + i * BUTTON_SIZE_X, option.rect.y(), BUTTON_SIZE_X, BUTTON_SIZE_Y)
                
                styledOption.option.rect = _rect

                styledOption.rect = _rect
                styledOption.option.icon = QIcon()
                styledOption.option.iconSize = QSize(15, 15)
                styledOption.option.state |= QStyle.State_Enabled
                # styledOption.features = QStyleOptionButton.DefaultButton
                # option.features = QStyleOptionButton.DefaultButton
                
                if i == 0:
                    if profile_state.find('cbrowser') != -1:
                        if profile_state.find('|') != -1:
                            profile_state = profile_state.split("|")[1]
                        if profile_state == "opening":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_opening_closing), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Opening"
                        elif profile_state == "closing":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_opening_closing), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Closing"
                        elif profile_state == "closed":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_closed), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                        elif profile_state == "opened":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_opened), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Close"
                        elif profile_state == "opened_other_device":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_disabled), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                        else:
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_closed), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                    else:
                        if profile_state.find('|') != -1:
                            profile_state = profile_state.split("|")[1]
                        if profile_state == "opening":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_opening_closing), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Opening"
                        elif profile_state == "closing":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_opening_closing), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Closing"
                        elif profile_state == "closed":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_closed), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                        elif profile_state == "opened":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_opened), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Close"
                        elif profile_state == "opened_other_device":
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_disabled), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                        else:
                            styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_firefox_closed), QIcon.Normal, QIcon.Off)
                            styledOption.option.text = "Open"
                elif i == 1:
                        styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_profile_edit), QIcon.Normal, QIcon.Off)
                        styledOption.option.text = "Edit"
                elif i == 2:
                        styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_profile_delete), QIcon.Normal, QIcon.Off)
                        styledOption.option.text = "Delete"
                painter.save()
                styledOption.style().drawControl(QStyle.CE_PushButton, styledOption.option, painter, styledOption)
                # QApplication.style().drawControl(QStyle.CE_PushButton, opt, painter, self.button)
                painter.restore()
            return

        super(StyledItemDelegate, self).paint(painter, option, index)


# Copyright (C) 2022 The Pyact Company Ltd.
# SPDX-License-Identifier: LicenseRef-Pyact-Commercial OR BSD-3-Clause
from __future__ import annotations

"""..site_packages.qtcompat port of the areachart example from qtmui v1.0"""

from typing import Optional, Union
import sys
from PySide6.QtCore import QPointF, Qt
from PySide6.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout
from PySide6.QtCharts import (QChart, 
                              QChartView, 
                              QLineSeries, 
                              QAreaSeries,
                              QBarCategoryAxis, 
                              QBarSeries, 
                              QBarSet, 
                              QChart,
                              QChartView, 
                              QValueAxis,
                              )
from PySide6.QtGui import QGradient, QPen, QLinearGradient, QPainter, QColor

from qtmui.material.styles import useTheme


class ChartArea(QWidget):
    def __init__(
                self,
                dir: str = "ltr", # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
                type: str = "line", # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
                series: object = None, 
                width: Optional[Union[str, int]] = None, 
                height: Optional[Union[str, int]] = None, 
                options: object = None,
                key: str = None,
                *args,
                **kwargs
                ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        self._type = type
        self._series = series
        self._width = width
        self._height = height
        self._options = options

        self._init_area_chart()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

 
    def _init_area_chart(self):
        theme = useTheme()

        self._line_series = []

        for i, item in enumerate(self._series):
            self._line_series.append(QLineSeries())
            for index, value in  enumerate(item.get("data") or item):
                if self._options.get("categories"):
                    self._line_series[i].append(QPointF(self._options["categories"][index], value))

        self.series = QAreaSeries(*self._line_series)

        self.series.setName("Batman")
        self.pen = QPen(QColor(theme.palette.primary.main))
        self.pen.setWidth(1)
        self.series.setPen(self.pen)

        self.gradient = QLinearGradient(QPointF(0, 0), QPointF(0, 1))
        self.gradient.setColorAt(0.0, QColor(theme.palette.primary.main))
        self.gradient.setColorAt(1.0, QColor(theme.palette.warning.main))
        self.gradient.setCoordinateMode(QGradient.ObjectBoundingMode)
        self.series.setBrush(self.gradient)

        self.chart = QChart()
        self.chart.addSeries(self.series)
        self.chart.setTitle("Simple areachart example")
        self.chart.createDefaultAxes()
        self.chart.axes(Qt.Orientation.Horizontal)[0].setRange(0, 20)
        self.chart.axes(Qt.Vertical)[0].setRange(0, 10)

        self._chart_view = QChartView(self.chart)
        # self._chart_view.setStyleSheet('background-color: transparent;')
        self._chart_view.setRenderHint(QPainter.RenderHint.Antialiasing)

        self.layout().addWidget(self._chart_view)

    def _set_stylesheet(self):
        theme = useTheme()

        text_color = QColor(theme.palette.text.secondary)
        # 1. Màu cho tiêu đề
        self.chart.setTitleBrush(text_color)

        # 2. Màu cho nhãn trục
        for axis in self.chart.axes():
            axis.setLabelsBrush(text_color)

        # 3. Màu cho nhãn trong legend (nếu bật)
        legend = self.chart.legend()
        legend.setLabelBrush(text_color)

        # Set the chart's background
        gradient = QLinearGradient(QPointF(0, 0), QPointF(0, 1))
        gradient.setColorAt(0.0, QColor(theme.palette.background.paper))  # Top color
        gradient.setColorAt(1.0, QColor(theme.palette.background.main))  # Bottom color
        gradient.setCoordinateMode(QGradient.ObjectBoundingMode)
        self.chart.setBackgroundBrush(gradient)
        # self.chart.setBackgroundBrush(QBrush(alpha(theme.palette.background.paper, 0.26)))
        self.chart.setBackgroundRoundness(10)  # Optional: Rounded edges for the background



from typing import Callable
from PySide6.QtCore import QSize, QCoreApplication, QEvent, Signal, QRect
from PySide6.QtGui import QMovie, QIcon, Qt, QColor, QBrush
from PySide6.QtWidgets import QStyledItemDelegate, QStyle, QApplication, QStyleOptionButton, QPushButton

from ..icon import icon_base_64_data, icon_chrome_closed, icon_chrome_opened, icon_chrome_opening_closing, \
    icon_profile_edit, icon_profile_delete, icon_chrome_disabled, icon_firefox_closed, icon_firefox_opened, icon_firefox_opening_closing, icon_firefox_disabled

from ...utils.icon import icon_base64_to_pixmap


from icon import *


class StyledItemDelegate(QStyledItemDelegate):

    mousePressedSignal = Signal(str)

    def __init__(self, 
                 parent=None,
                 column: int = None,
                 styledOption: object = None,
                 onMousePressed: Callable = None
                 ):
        super(StyledItemDelegate, self).__init__(parent)
        if onMousePressed is not None:
            self.mousePressedSignal.connect(onMousePressed)
        self.column = column
        self.styledOption = styledOption


    def split_rect(self, rect, num_divisions):
        # Tính toán chiều rộng của mỗi phần chia
        width_per_division = rect.width() // num_divisions
        
        # Tạo danh sách chứa các hình chữ nhật con
        sub_rects = []
        
        # Tạo các hình chữ nhật con
        for i in range(num_divisions):
            if i == 0:
                sub_rect = QRect((rect.left() + i * width_per_division) - 1, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
            if i == 1:
                sub_rect = QRect(rect.left() + i * width_per_division, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
            if i == 2:
                sub_rect = QRect((rect.left() + i * width_per_division) + 1, rect.top(), width_per_division + 1, rect.height() - 4)
                sub_rects.append(sub_rect)
        return sub_rects

    def point_in_rects(self, point, rects):
        for i, rect in enumerate(rects):
            if rect.contains(point):
                return i
        return -1

    def editorEvent(self, event, model, option, index):
        sub_rects = self.split_rect(option.rect, 3)
        test_point = event.pos()

        # print('sub_rects___', sub_rects)

        # Kiểm tra xem test_point có nằm trong hình chữ nhật nào không
        result = self.point_in_rects(test_point, sub_rects)

        if option.state & QStyle.State_MouseOver:
            print('eoooooooooooooooooooooooooooeeeeeeeeeeee')
            option.backgroundBrush = QBrush(QColor(211, 211, 211))  # LightGray color

        if index.column() == 10 and event.type() == QEvent.MouseButtonRelease:
            if result != -1:
                if result == 0:
                    self.mousePressedSignal.emit(f"openclose_{index.row()}_{index.column()}")
                if result == 1:
                    self.mousePressedSignal.emit(f"edit_{index.row()}_{index.column()}")
                if result == 2:
                    self.mousePressedSignal.emit(f"delete_{index.row()}_{index.column()}")
                return True
            else:
                print("The point is not in any sub-rectangle.")

        return super().editorEvent(event, model, option, index)

    def paint(self, painter, option, index):
        if index.column() == self.column:  # Assuming the button column is the third column
            # buttons = [("Open", "Blac.png"), ("Edit", "Blac.png"), ("Delete", "Blac.png")]  # Define your button labels and icons
            
            self.styledOption.option.rect = option.rect

            self.styledOption.rect = option.rect
            self.styledOption.option.icon = QIcon()
            self.styledOption.option.iconSize = QSize(15, 15)
            self.styledOption.option.state |= QStyle.State_Enabled
            # styledOption.features = QStyleOptionButton.DefaultButton
            # option.features = QStyleOptionButton.DefaultButton
            
            self.styledOption.option.icon.addPixmap(icon_base64_to_pixmap(icon_chrome_opening_closing), QIcon.Normal, QIcon.Off)
            self.styledOption.option.text = "Opening"
            painter.save()
            self.styledOption.style().drawControl(QStyle.CE_PushButton, self.styledOption.option, painter, self.styledOption)
            painter.restore()
            return

        super(StyledItemDelegate, self).paint(painter, option, index)


from typing import Callable, Dict, Optional, Union
import uuid
from PySide6.QtWidgets import QSplitter, QTextEdit, QSizePolicy
from PySide6.QtCore import Qt

from qtmui.hooks import State, useEffect
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

class Splitter(QSplitter):
    def __init__(
                self, 
                orientation=Qt.Horizontal, 
                parent=None, 
                min_width=None, 
                max_width=None, 
                min_height=None, 
                max_height=None,
                handleWidth: int = 0,
                children: list= None,
                sx: Optional[Union[Callable, str, Dict]]= None
                ):
        super().__init__(orientation, parent)

        self._handleWidth = handleWidth
        self._childrend = children
        self._min_width = min_width
        self._max_width = max_width
        self._min_height = min_height
        self._max_height = max_height
        self._sx = sx

        self._init_ui()


    def _init_ui(self):
        self.setObjectName("PySplitter")
        self.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        # self.setMinimumWidth(QSizePolicy.Maximum, QSizePolicy.Expanding)

        self.setHandleWidth(self._handleWidth)
        
        self.set_min_width(self._min_width)
        self.set_max_width(self._max_width)
        self.set_min_height(self._min_height)
        self.set_max_height(self._max_height)

        if self._childrend is not None:
            for item in self._childrend:
                item.get("widget").setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred))
                # item.get("widget").setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)
                if item.get("sizePolicy") == "expanding":
                    item.get("widget").setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred))
                if item.get("maximumWidth") is not None:
                    item.get("widget").setMaximumWidth(item.get("maximumWidth"))
                if item.get("minimumWidth") is not None:
                    item.get("widget").setMinimumWidth(item.get("minimumWidth"))
                self.addWidget(item.get("widget"))


        self.theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [self.theme.state]
        )
        self._set_stylesheet()

    def _set_stylesheet(self):
        theme = useTheme()
        component_styles = theme.components

        PySplitter_root = component_styles[f"PySplitter"].get("styles")["root"]
        PySplitter_root_qss = get_qss_style(PySplitter_root)

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx)
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx)
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        self.setStyleSheet(
            f"""
                #PySplitter {{
                    {PySplitter_root_qss}
                    {sx_qss}
                }}
            """
        )


    def set_min_width(self, min_width):
        if min_width is not None:
            self.setMinimumWidth(min_width)

    def set_max_width(self, max_width):
        if max_width is not None:
            self.setMaximumWidth(max_width)

    def set_min_height(self, min_height):
        if min_height is not None:
            self.setMinimumHeight(min_height)

    def set_max_height(self, max_height):
        if max_height is not None:
            self.setMaximumHeight(max_height)

    def add_pages(self, num_pages):
        for _ in range(num_pages):
            text_edit = QTextEdit()
            self.addWidget(text_edit)

